<?php
/**
 * MiNBank business object: account
 *
 * $Source: /cvsroot/minbank/minba/minba/bo/babo_account.php,v $
 * $Author: eltres $
 * $Date: 2007/01/21 23:18:32 $
 * $Revision: 1.14 $
 */

require_once("../bagn_config.php");
require_once("$appl_path/bo/babo_globmeth.php");
require_once("$minsoft_path/utility/utdb_access.php");
require_once("$minsoft_path/utility/utgn_config.php");
require_once("$minsoft_path/utility/utgn_trace.php");
require_once("$minsoft_path/utility/utgn_fault.php");
require_once("$minsoft_path/utility/utgn_message.php");
require_once("$minsoft_path/utility/utrl_role.php");
require_once("$minsoft_path/utility/utgn_checkit.php");
require_once("$minsoft_path/utility/utxr_client.php");

global $girokonto;

/**
 * Accounts, centralbank accounts and transfers.
 *
 * account attributes:
 * <table>
 * <tr><td>kindofaccount</td><td>kind of account, 0=giro</td></tr>
 * <tr><td>bankid</td><td>bank id of bank managing the account</td></tr>
 * <tr><td>accountid</td><td>id of account</td></tr>
 * <tr><td>accountowner</td><td>customerid of accountowner</td></tr>
 * <tr><td>accountname</td><td>text describing the account</td></tr>
 * <tr><td>accountcurrency</td><td>currency of account</td></tr>
 * <tr><td>accountbalance</td><td>current balance of account</td></tr>
 * <tr><td>lastfeedate</td><td>month of last charge of fees</td></tr>
 * <tr><td>createdate</td><td>date the account has been created</td></tr>
 * <tr><td>deletedate</td><td>date the account has been deleted</td></tr>
 * <tr><td>changedate</td><td>date the account has been deleted</td></tr>
 * <tr><td>deleted</td><td>deletion flag</td></tr>
 * <tr><td>locked</td><td>locked flag</td></tr>
 * <tr>
 * </table>
 *
 * transfer attributes:
 * <table>
 * <tr><td>date</td><td>date of transfer execution</td></tr>
 * <tr><td>remitterbankid</td><td>bank id of remitter</td></tr>
 * <tr><td>remitteraccountid</td><td>account id of remitter</td></tr>
 * <tr><td>payeebankid</td><td>bankid of payee</td></tr>
 * <tr><td>payeeaccountid</td><td>account id of payee</td></tr>
 * <tr><td>customerid</td><td>customer id of payee</td></tr>
 * <tr><td>purpose</td><td>purpose text of transfer</td></tr>
 * <tr><td>currencyid</td><td>currency of transfer</td></tr>
 * <tr><td>amount</td><td>amount transferred</td></tr>
 * <tr><td>payeename</td><td>name of payee</td></tr>
 * </table>
 */
class Account
{
    var $standingOrder=false; // temporary standing order flag for standing order transfers
    var $chargeOrder=false; // temporary standing order flag for charges
    
    //*****************   private tool functions ****************************

    /**
     * Gets a word by a name depending on the language.
     *
     * @param $lang language of word
     * @param $name name of word
     */
    function getKeyword($lang, $name) {
        global $minsoft_path; 
        require("$minsoft_path/minba/msg/keywords_".$lang.".txt");
        return $keywords[$name];
    }

    /**
     * Checks the user input of transfers.
     *
     * @access private
     * @param $lang language for messages
     * @param $transfer transfer attribute array
     * @return fault or nothing=OK
     */
    function check_transfer_params($lang,$transfer)
    {
        $check = array(
                "remitteraccountid"=>2,
                "remitterbankid"=>2,
                "payeeaccountid"=>2,
                "payeebankid"=>2,
                "accountcurrency"=>1,
                "amount"=>3,
                "purpose"=>1);

        foreach ($check as $checkname => $checkitem)
        {
            switch($checkitem)
            {
                case 1: // alphanumeric attribute
                    $item = $transfer["$checkname"];
                    if($item == "")
                    {
                        return getFault("2053",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                        return getFault("2005",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    break;
                case 2: // numeric attribute
                    $item = $transfer["$checkname"];
                    if(! is_numeric($item) || intval($item) != $item )
                    {
                        return getFault("2006",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    break;
                case 3: // amount
                    $item = $transfer["$checkname"];
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    if($item < 0)
                        return getFault("2077");
                    break;
                default:
                    return getFault("2007");
                    break;
            }
        }
    }

    /**
     * Checks the user input of an account.
     *
     * @access private
     * @param $account account attriubute array
     * @param $lang language for message
     * @return fault or nothing=OK
     */
    function check_account_params($lang, $account)
    {
        $result = NULL;
        $check = array(
                "kindofaccount" =>2,
                "accountowner" => 1,
                "accountcurrency"=>1,
                "bankid"=>2);

        foreach ($check as $checkname => $checkitem)
        {
            switch($checkitem)
            {
                case 1: // alpha numeric attribute
                    $item = $account["$checkname"];
                    if($item == "")
                    {
                        return getFault("2053",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                        return getFault("2005",$dname["$checkname"]);
                        break;
                    }
                    break;
                case 2: // numeric attribute
                    $item = $account[$checkname];
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$dname["$checkname"]);
                        break;
                    }
                    break;
                default:
                    return getFault("2007");
                    break;
            }
        }
        return $result;
    }

    /**
     * Checks the central bank account change parameters.
     *
     * @access private
     * @param $lang language for messages
     * @param $accountparams central bank account amount change parameters
     * @return fault or nothing=OK
     */
    function check_centralaccount($lang,$accountparams)
    {
        $centralbankid = $accountparams["centralbankid"];
        $centralaccountid = $accountparams["centralaccountid"];
        $centralamount = $accountparams["centralamount"];

        if(!($centralbankid != "" && is_numeric($centralbankid)))
        {
            return getFault("2006", $this->getKeyword($lang, "bankid"));
        }
        if(!($centralaccountid != "" && is_numeric($centralaccountid)))
        {
            return getFault("2006", $this->getKeyword($lang, "stateaccountnumber"));
        }
        if(!($centralamount != "" && is_numeric($centralamount)))
        {
            return getFault("2006", $this->getKeyword($lang, "amountchange"));
        }

    }

    /**
     * Checks if it is a bankmanager, owner, account deputee or customer deputee.
     *
     * @access private
     * @param $lang language for messages
     * @param $accountid id of account
     * @param $bankid id of bank
     * @param $us_id user id
     * @param $us_role user role
     * @param $prefix database table name prefix
     * @param $lang language
     * @param $account account transfer parameters
     * @return fault or empty=OK
     */
    function check_authorities($lang,$accountid,$bankid,$us_id,$us_role,$prefix)
    {
        $auth_check_db = &new DBAccess;
        $result = NULL;

        // get the account owner
        $select_owner = "SELECT accountowner "
                ."FROM ".$prefix."account "
                ."WHERE accountid = '".$accountid."' "
                ."AND bankid ='".$bankid."' "
                ."AND deleted = '0'";

         $auth_check_db->query($select_owner);
         if($auth_check_db->errno != 0)
         {
            return $auth_check_db->getError();
         }

         if($auth_check_db->num_rows() != "1")
         {
             //account does not exist
             return getFault("2016");
        }

        $auth_check_db->next_record();
        $accountowner = $auth_check_db->Record["accountowner"];

        // check if account id is a deputee or customer deputee
        if(!($us_role & utrl_bankmanager) && ! $this->standingOrder)
        {
            if($us_id != $accountowner)
            {
                $select_deputee = "SELECT deputeeid "
                        ."FROM ".$prefix."deputeeaccount "
                        ."WHERE accountid = '".$accountid."' "
                        ."AND deputeeid = '".$us_id."' "
                        ."AND deleted = '0'";

                $auth_check_db->query($select_deputee);
                if($auth_check_db->errno != 0)
                {
                    return $auth_check_db->getError();
                }

                if($auth_check_db->num_rows() == "0")
                {
                    // check if customer is a deputee ?
                    $select_authcust = "SELECT deputeeid "
                            ."FROM ".$prefix."deputeecust "
                            ."WHERE deputeeid ='".$us_id."' "
                            ."AND bankid = '".$bankid."' "
                            ."AND deleted = '0' "
                            ."AND customerid = '".$accountowner."'";

                    $auth_check_db->query($select_authcust);
                    if($auth_check_db->errno != 0)
                    {
                        $result    = $auth_check_db->getError();
                        return $result;
                    }
                    if($auth_check_db->num_rows()== 0)
                    {
                        return getFault("2016");
                    }
                 }
            }
        }
        return $result;
    }

    //*************    public class methods *************************

    /**
     * Creates an account.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $account account attributes
     * @return fault or new account id
     */
    function create($secId,$lang,$account)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }

        $result    = $this->check_account_params($lang,$account,$us_id);
        if(isFault($result))
        {
            return $result;
        }
        $account_create    = &new DBAccess;

        $kindofaccount   = $account["kindofaccount"];
        $accountname     = $account["accountname"];
        $accountowner    = $account["accountowner"];
        $accountcurrency = $account["accountcurrency"];
        $accountbalance  = 0.00;
        $bankid          = $account["bankid"];

        // customerid exists? and is active?
        $result= get_custid($accountowner,$bankid);

        if($result != 1)
        {
            return getFault(2014);
        }

        // a customer administers only his own accounts
        // a bankmanager administers all accounts belonging to the bank
        if(!($us_role & (utrl_bankmanager | utrl_customer)))
        {
            return getFault(2024);
        }

        // insert into account table
        $insert_account    = "INSERT INTO ".$cf_db_prefix."account "
                ."(kindofaccount,"
                ."accountowner,"
                ."accountname,"
                ."accountcurrency,"
                ."accountbalance,"
                ."bankid) "
                ."VALUES "
                ."('".$kindofaccount."',"
                ."'".$accountowner."',"
                ."'".$accountname."',"
                ."'".$accountcurrency."',"
                ."'".$accountbalance."',"
                ."'".$bankid."')";

        $account_create->query($insert_account);
        if($account_create->errno != 0)
        {
            return $account_create->getError();
        }

        // account id will be delivered by autoincrement
        // the mysql function "max" does not work!!!
        $accountid = mysql_insert_id();
        return $accountid;
    }
    
    /**
     * Updates an account.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $account account attributes
     * @return fault or nothing
     */
    function update($secId,$lang,$account)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if(!($us_role & (utrl_bankmanager | utrl_customer | utrl_centralbankmanager)))
        {
            return getFault("2008");
        }

        $result    = $this->check_account_params($lang,$account,$us_id);
        if(isFault($result))
        {
            return $result;
        }
        $account_update = &new DBAccess;

        $accountname     = $account["accountname"];
        $accountid       = $account["accountid"];
        $bankid          = $account["bankid"];
        $accountowner    = $account["accountowner"];

        // customerid exists? and is active?
        $result= get_custid($accountowner,$bankid);

        if($result != 1)
        {
            return getFault(2014);
        }

        // update account table
        $sql = "UPDATE ".$cf_db_prefix."account SET "
                ."accountname='".$accountname."',"
                ."changedate='".time()."' "
                ."WHERE accountid='".$accountid."' "
                ."AND bankid='".$bankid."'";

        $account_update->query($sql);
        
        if($update_create->errno != 0)
        {
            return $account_create->getError();
        }
    }
    
    /**
     * Reads the account information.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid id of account
     * @param $bankid id of bank
     * @return fault or array of account attributes
     */
    function read($secId,$lang,$accountid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & (utrl_bankmanager | utrl_customer | utrl_centralbankmanager)))
        {
            return getFault("2008");
        }

        if($accountid == "0" || ! is_numeric($accountid))
        {
            $result    = "2016";
            return getFault($result);
        }

        $account_read_db = &new DBAccess;

        // customer or deputee valid?
        $result    = $this->check_authorities($lang,$accountid,$bankid,$us_id,$us_role,$cf_db_prefix);

        if(isFault($result))
        {
            return $result;
        }

        // select account parameter values
        $select_account    = "SELECT accountid,"
                        ."kindofaccount,"
                        ."accountname,"
                        ."accountowner,"
                        ."accountcurrency,"
                        ."accountbalance,"
                        ."createdate,"
                        ."deletedate,"
                        ."changedate,"
                        ."deleted "
                        ."FROM ".$cf_db_prefix."account "
                        ."WHERE accountid    = '".$accountid."' "
                        ."AND bankid = '".$bankid."' "
                        ."AND deleted = '0'";


        $account_read_db->query($select_account);

        if($account_read_db->errno != 0)
        {
            return $account_read_db->getError();
        }
        if($account_read_db->num_rows() != "1")
        {
            $result = 2016;
            return getFault($result);
        }
        else
        {

            // fill array with account information
            $account_read_db->next_record();
            $deleted = $account_read_db->Record['deleted'];
            if($deleted != 0)
            {
                return getFault(2021);
            }
            else
            {
                $account['accountid']       = $account_read_db->Record['accountid'];
                $account['kindofaccount']   = $account_read_db->Record['kindofaccount'];
                $account['accountname']     = $account_read_db->Record['accountname'];
                $account['accountowner']    = $account_read_db->Record['accountowner'];
                $account['accountcurrency'] = $account_read_db->Record['accountcurrency'];
                $account['accountbalance']  = $account_read_db->Record['accountbalance'];
                $account['createdate']      = $account_read_db->Record['createdate'];
                $account['deletedate']      = $account_read_db->Record['deletedate'];
                $account['changedate']      = $account_read_db->Record['changedate'];
                $account['deleted']         = $account_read_db->Record['deleted'];
            }
        }
        return    $account;
    }

    /**
     * Reads the account owner id and name.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid id of account
     * @param $bankid id of bank
     * @return fault or array with customerid and customername
     */
    function get_accountowner($secId,$lang,$accountid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        if(!($accountid != "0" && is_numeric($accountid)))
        {
            $result = "2016";
            return getFault($result);
        }

        $account_read_db = &new DBAccess;

        // select customer parameter values
        $select_customer = "SELECT b.customerid, b.customername, a.deleted "
                ."FROM ".$cf_db_prefix."account AS a, ".$cf_db_prefix."customer AS b "
                ."WHERE a.accountid    = '".$accountid."' "
                ."AND a.bankid = '".$bankid."' "
                ."AND a.bankid = b.bankid "
                ."AND a.deleted = '0' "
                ."AND a.accountowner = b.customerid";

        $account_read_db->query($select_customer);

        if($account_read_db->errno != 0)
        {
            return $account_read_db->getError();
        }
        if($account_read_db->num_rows()    != "1")
        {
            $result    = 2016;
            return getFault($result);
        }
        else
        {

            // fill array with customer information
            $account_read_db->next_record();
            $deleted = $account_read_db->Record['deleted'];
            if($deleted    != 0)
            {
                return getFault(2021);
            }
            else
            {
                $customer[]= array("customerid" => $account_read_db->Record['customerid'],
                                  "customername" => $account_read_db->Record['customername']);
            }
        }
        return $customer;
    }

    /**
     * Gets the account deputees.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid id of account
     * @param $bankid id of bank
     * @return fault or array of account authority attributes
     */
    function account_authority($secId,$lang,$accountid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }

        if(!($accountid    != "0" && is_numeric($accountid)))
        {
            $result    = "2016";
            return getFault($result);
        }

        $account_authority_db = &new    DBAccess;

        $select_account_authority =
                "SELECT a.accountid, a.bankid, a.deputeeid, b.customername "
                ."FROM ".$cf_db_prefix."deputeeaccount AS a, ".$cf_db_prefix."customer AS b "
                ."WHERE a.accountid    = '".$accountid."' "
                ."AND a.bankid = '".$bankid."' "
                ."AND a.bankid = b.bankid "
                ."AND a.deputeeid = b.customerid "
                ."AND a.deleted = '0'";

        $account_authority_db->query($select_account_authority);
        if($account_authority_db->errno != 0)
        {
            return $account_authority_db->getError();

        }

        if($account_authority_db->num_rows() == "0")
        {
            $result    = 2042;
            return getFault($result);
        }
        else
        {
            // fill list of transfers in an array
            while ($account_authority_db->next_record())
            {
                $account_authority[] = array(
                "accountid"   => $account_authority_db->Record["accountid"],
                "bankid"      => $account_authority_db->Record["bankid"],
                "deputeeid"   => $account_authority_db->Record["deputeeid"],
                "deputeename" => $account_authority_db->Record["customername"]);

            }
        }

        return $account_authority;
    }

   /**
     * Deletes an authority member of an account.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid id of account
     * @param $accountowner id of account owner
     * @param $bankid id of bank
     * @param $deputeeid id of deputee
     * @return fault or nothing
     */
    function account_authority_delete($secId,$lang,$accountid,
            $accountowner,$bankid,$deputeeid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }

        if(!($accountid    != "0" && is_numeric($accountid)))
        {
            $result    = "2016";
            return getFault($result);
        }

        $account_authority_db = &new    DBAccess;

        $select_account_authority    = "SELECT accountid,bankid,deputeeid "
                    ."FROM ".$cf_db_prefix."deputeeaccount "
                    ."WHERE accountid    = '".$accountid."' "
                    ."AND bankid = '".$bankid."' "
                    ."AND deputeeid = '".$deputeeid."' "
                    ."AND deleted = '0'";

        $account_authority_db->query($select_account_authority);
        if($account_authority_db->errno != 0)
        {
            $result = $account_authority_db->errno;
            return getFault($result);
        }

        if($account_authority_db->num_rows() == "0")
        {
            $result    = 2016;
            return getFault($result);
        }

        // customer authorsation?
        $account_authority_db->next_record();
        $deputeeid = $account_authority_db->Record["deputeeid"];
        if(!($us_role & utrl_bankmanager))
        {
            if(($us_id != $accountowner)&&($us_id != $deputeeid))
            {
                // check customer deputee
                $select_authcust = "SELECT deputeeid "
                        ."FROM ".$cf_db_prefix."deputeecust "
                        ."WHERE deputeeid ='".$us_id."' "
                        ."AND $bankid = '".$bankid."' "
                        ."AND deleted    = '0' "
                        ."AND customerid = '".$accountowner."'";

                $account_authority_db->query($select_authcust);
                if($account_authority_db->errno != 0)
                {
                    $result = $account_authority_db->errno;
                    return getFault($result);
                }

                if($account_authority_db->num_rows() == "0")
                {
                    return getFault(2025);
                }
            }
        }

        // set delete sign
        $delete_account = "UPDATE ".$cf_db_prefix."deputeeaccount "
                ."SET deleted = '1' "
                ."WHERE accountid    ='".$accountid."' "
                ."AND bankid = '".$bankid."' "
                ."AND deputeeid = '".$deputeeid."'";

        $account_authority_db->query($delete_account);
        if($account_authority_db->errno != 0)
        {
            return getFault($account_authority_db->errno);
        }
    }

    /**
     * Inserts an authority member of an account.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid id of account
     * @param $accountowner id of account owner
     * @param $bankid id of bank
     * @param $deputeeid id of deputee
     * @return fault or nothing
     */
    function account_authority_new($secId,$lang,$accountid,
            $accountowner,$bankid,$deputeeid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        if($us_id == $deputeeid || $deputeeid == $accountowner)
        {

            $result = "2043";
            return getFault($result);
        }
        if(!($accountid    != "0" && is_numeric($accountid)))
        {
            $result    = "2016";
            return getFault($result);
        }

        $account_authority_db = &new    DBAccess;

        $select_customer = "SELECT customerid "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid = '".$deputeeid."'";

        $account_authority_db->query($select_customer);
        if($account_authority_db->errno != 0)
        {
            return $account_authority_db->getError();
        }

        if($account_authority_db->num_rows() == "0")
        {
            return getFault("2047");
        }
        $select_account_authority    = "SELECT accountid,bankid,deputeeid,deleted "
                ."FROM ".$cf_db_prefix."deputeeaccount "
                ."WHERE accountid    = '".$accountid."' "
                ."AND bankid = '".$bankid."' "
                ."AND deputeeid = '".$deputeeid."'";

        $account_authority_db->query($select_account_authority);
        if($account_authority_db->errno != 0)
        {
            return $account_authority_db->getError();

        }
        if($account_authority_db->num_rows() == "0")
        {
            // insert into deputeeaccount table
            $insert_account_authority    = "INSERT into {$cf_db_prefix}deputeeaccount "
                ."(accountid,bankid,deputeeid,deleted) "
                ."VALUES('".$accountid."','".$bankid."','".$deputeeid."','0')";

            $account_authority_db->query($insert_account_authority);
            if($account_authority_db->errno != 0)
            {
                return $account_authority_db->getError();
            }
        }
        else
        {
            $account_authority_db->next_record();
            if($account_authority_db->Record['deleted'] == '0')
            {
                $result = "2044";
                return getFault($result);
            }
            else
            {
                // reset delete sign
                $update_account = "UPDATE {$cf_db_prefix}deputeeaccount "
                        ."SET deleted ='0' "
                        ."WHERE accountid ='".$accountid."' "
                        ."AND    bankid = '".$bankid."' "
                        ."AND deputeeid = '".$deputeeid."'";

                $account_authority_db->query($update_account);
                if($account_authority_db->errno != 0)
                {
                    return getFault($account_authority_db->errno);
                }
            }
        }
    }

    /**
     * Displays the account history of transfers.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid id of account
     * @param $bankid id of bank
     * @return fault or array of arrays of account history data
     */
    function account_extract($secId,$lang,$accountid,$bankid,$fromdate=0,$todate=2000000000)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }

        if(!($accountid    != "0" && is_numeric($accountid)))
        {
            $result    = "2016";
            return getFault($result);
        }

        $account_extract_db = &new    DBAccess;

        // customer or deputee valid?
        $result    = $this->check_authorities($lang,$accountid,
                $bankid,$us_id,$us_role,$cf_db_prefix);

        if(isFault($result))
        {
            return $result;
        }

        $select_account_extract = "SELECT date,remitteraccountid,remitterbankid,"
                ."payeeaccountid,payeebankid,currencyid,amount,purpose "
                ."FROM ".$cf_db_prefix."transfer "
                ."WHERE ((remitteraccountid = '".$accountid."' "
                ."AND remitterbankid = '".$bankid."') "
                ."OR (payeeaccountid = '".$accountid."' "
                ."AND payeebankid = '".$bankid."')) "
                ."AND (date BETWEEN '".$fromdate."' AND '".$todate."') "
                ."ORDER BY date";

        $account_extract_db->query($select_account_extract);

        if($account_extract_db->errno != 0)
        {
            return $account_extract_db->getError();
        }

        if($account_extract_db->num_rows()    == "0")
        {
            $result    = 2033;
            return getFault($result);
        }
        else
        {
            // fill list of transfers in an array
            while ($account_extract_db->next_record())
            {

                $debitaccountid = $account_extract_db->Record["remitteraccountid"];
                $debitbankid = $account_extract_db->Record["remitterbankid"];
                $favoraccountid = $account_extract_db->Record["payeeaccountid"];
                $favorbankid = $account_extract_db->Record["payeebankid"];
                $date = $account_extract_db->Record["date"];
                $amount = $account_extract_db->Record["amount"];
                $favoramount = 0;
                $debitamount= 0;
                if( ($debitaccountid == $accountid) && ($debitbankid == $bankid))
                {
                    $debitamount = $amount;
                }
                if( ($favoraccountid == $accountid) && ($favorbankid == $bankid))
                {
                    $favoramount = $amount;
                }
                $valdate = date ("d.m.Y H:i:s", $date);
                $account_extract[] = array(
                    "date"    =>  $valdate,
                    "purpose" => $account_extract_db->Record['purpose'],
                    "accountcurrency" =>$account_extract_db->Record['currencyid'],
                    "debitamount" => $debitamount,
                    "favoramount" => $favoramount);
            }
        }
        return $account_extract;
    }


    /**
     * Delete an account.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid id of account
     * @param $bankid id of bank
     * @return fault or nothing
     */
    function delete($secId,$lang,$accountid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        // check security
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }

        $account_delete    = &new DBAccess;

        // account exists? account has accountbalance=0?
        $select_account    = "SELECT accountid,accountowner,accountbalance,bankid "
                ."FROM ".$cf_db_prefix."account "
                ."WHERE accountid ='".$accountid."' "
                ."AND bankid = '".$bankid."' "
                ."AND deleted    = '0'";

        $account_delete->query($select_account);
        if($account_delete->errno != 0)
        {
            $result    = $account_delete->getError();
            return $result;
        }
        if($account_delete->num_rows() == 0)
        {
            return getFault(2016);
        }

        // customer or deputee valid?
        $result    = $this->check_authorities($lang,$accountid,
                $bankid,$us_id,$us_role,$cf_db_prefix);

        if(isFault($result))
        {
            return $result;
        }

        // account amount not zero?
        $account_delete->next_record();
        if($account_delete->Record["accountbalance"] > 0)
        {
            return getFault(2030);
        }

        // set delete sign
        $delete_account = "UPDATE ".$cf_db_prefix."account "
                ."SET deleted ='1' "
                ."WHERE accountid ='".$accountid."' "
                ."AND bankid = '".$bankid."'";

        $account_delete->query($delete_account);
        if($account_delete->errno != 0)
        {
            return $account_delete->getError();
        }
    }

    /**
     * Checks the accountid and customerid of remittee and transfer amount.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $transfer account transfer parameters
     * @return fault or nothing
     */
    function transfer_payee_international($secId,$lang,$transfer)
    {
      global $cf_db_prefix;

      $transfer0=$transfer;
      $db = &new DBAccess;
      $sql = "SELECT * "
	."FROM ".$cf_db_prefix."centralbank ";
      $db->query($sql);
      $db->next_record();
      $bankid=$db->Record['reservebankid'];
      $accountid=$db->Record['reserveaccountid'];
      $transfer0['payeebankid']=$bankid;
      $transfer0['payeeaccountid']=$accountid;
      $result = $this->transfer_payee($secId,$lang,$transfer0);
      if ($result!=0)
	{
	  return $result;
	}
      $bankid=$transfer['payeebankid'];
      $ibic=$bankid[0].$bankid[1].$bankid[2];
      $bankid=$bankid-$ibic*1000;
      $db = &new DBAccess;
      $sql = "SELECT * "
	."FROM ".$cf_db_prefix."inter "
	."WHERE ibic='".$ibic."'";
      $db->query($sql);
      if($db->errno != 0)
        {
	  return $db->getError();
        }
      if ($db->num_rows() != 1)
	{
	  return getFault("2010");
	}
      $db->next_record();
      $server=$db->Record['server'];
      $user=$db->Record['user'];
      $password=$db->Record['password'];
      $transfer['payeebankid']=$bankid;
      $transfer['remitterbankid']=$db->Record['accountbank'];
      $transfer['remitteraccountid']=$db->Record['account'];
      $transfer['amount']=round($db->Record['rate']*$transfer['amount'],2);
      $error = false;
      // make a client object
      $client = new UTXRClient;
      // set remote interface for user object
      $client->setServer($server, "/user/ri_xmlrpc/usri_server.php");
      // call remote interface for login
      $params = array(1, $user, $password); // language, userid, password
      $response = $client->call("user.login", $params);
      // check if an error occured
      
      if(isFault($response)) {
	$error = true;
	   echo "<p><b>FEHLER: ".$response[0]["faultString"]."</b></p>\n";
	return getFault("2081");
      }
      if(! $error)
	{
	  // get security id from login
	  $secId = $response[1];
	  // set remote interface for bank object
	  $client->setServer($server, "/minba/ri_xmlrpc/bari_transfer.php", 80);
	  $param=array($secId,1,$transfer); // security-id,language,accountid,bankid
	  $response = $client->call("Account.transfer_process",$param);
	}
      if(isFault($response)) {
	$error = true;
	echo "<p><b>FEHLER: ".$response[0]["faultString"]."</b></p>\n";
	return getFault("2082");
      }
    }

    /**
     * Checks the accountid and customerid of remittee and transfer amount.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $transfer account transfer parameters
     * @return fault or nothing
     */
    function transfer_payee($secId,$lang,$transfer)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if((!($us_role & utrl_bankmanager || $us_role & utrl_customer))
                && (!($this->standingOrder) && ($us_role & utrl_technicaluser)))
        {
            return getFault(2008);
        }

         // check User Input
        $result = $this->check_transfer_params($lang,$transfer);
        if(isFault($result))
        {
            return $result;
        }

        $account_transfer_db = &new DBAccess;

        $payeeaccountid = $transfer["payeeaccountid"];
        $amount =$transfer["amount"];
        $payeebankid = $transfer["payeebankid"];
        $remitteraccountowner = $transfer["remitteraccountowner"];

        // customer exists?
        $select_account = "SELECT accountid, accountbalance "
                ."FROM ".$cf_db_prefix."account "
                ."WHERE accountid = '".$payeeaccountid."' "
                ."AND bankid = '".$payeebankid."' "
                ."AND deleted = '0'";
        $account_transfer_db->query($select_account);

        if($account_transfer_db->errno != 0)
        {
            return $account_transfer_db->getError();
        }

        if($account_transfer_db->num_rows() != "1")
        {
            return 2016;
        }
        else
        {
            // Transfer amount to payee account
            $account_transfer_db->next_record();
            $rec_accountbalance = $account_transfer_db->Record['accountbalance'];
            $rec_accountbalance = $rec_accountbalance + $amount;
            
            $update_account = "UPDATE ".$cf_db_prefix."account "
                    ."SET accountbalance ='".$rec_accountbalance."' "
                    ."WHERE accountid = '".$payeeaccountid."' "
                    ."AND bankid = '".$payeebankid."'";
            $account_transfer_db->query($update_account);
            if($account_transfer_db->errno != 0)
            {
                return $account_transfer_db->getError();
            }
        }
    }

    /**
     * Checks the accountid and customer of remitter to see if they exist and
     * have enough money.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $transfer account transfer parameters
     * @return fault or 0 or nothing
     */
    function transfer_remitter($secId,$lang,$transfer)
    {
        global $girokonto;
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if((!($us_role & utrl_bankmanager || $us_role & utrl_customer))
                && (!($this->standingOrder) && ($us_role & utrl_technicaluser)))
        {
            return getFault("2008");
        }
        $result = $this->check_transfer_params($lang,$transfer);
        if(isFault($result))
        {
            return $result;
        }
        $account_transfer_db = &new DBAccess;

        $remitteraccountid = $transfer["remitteraccountid"];
        $remitterbankid = $transfer["remitterbankid"];
        $accountcurrency = $transfer["accountcurrency"];
        $amount =$transfer["amount"];
        $remitteraccountowner = $transfer["remitteraccountowner"];
        $purpose = $transfer["purpose"];
            
        // account exists? customer has the authorisation?
        $select_account = "SELECT accountid,kindofaccount,accountowner,accountcurrency,accountbalance "
                ."FROM ".$cf_db_prefix."account "
                ."WHERE accountid='".$remitteraccountid."' "
                ."AND bankid='".$remitterbankid."'";

        $account_transfer_db->query($select_account);
        if($account_transfer_db->errno != 0)
        {
            return $account_transfer_db->getError();
        }
        if($account_transfer_db->num_rows() != "1")
        {
            $result = 2016;
            return getFault($result);
        }
        else
        {
            // accountowner valid?
            $account_transfer_db->next_record();
            $rec_accountowner = $account_transfer_db->Record['accountowner'];
            $rec_accountcurrency =
            $account_transfer_db->Record['accountcurrency'];
            $rec_accountbalance =
            $account_transfer_db->Record['accountbalance'];
            $rec_kindofaccount=
            $account_transfer_db->Record['kindofaccount'];

            // customer or deputee valid?
            $result = $this->check_authorities($lang,$remitteraccountid,
                    $remitterbankid,$us_id,$us_role,$cf_db_prefix);
                    
            if(isFault($result))
            {
                return $result;
            }
            else
            {
                // enough money on the remitter account?
                if(!$this->chargeOrder) {
                    $rec_accountbalance = $rec_accountbalance - $amount;
                    if((double)$rec_accountbalance < 0.00)
                    {
                        return getFault("2018");
                    }
                }
                
                //amount zero?
                if((double)$amount == 0.00)
                {
                    return getFault("2051");
                }

                // is it a giro account?
                else
                {
                    if($rec_kindofaccount != $girokonto)
                    {
                        return getFault("2019");
                    }
                    else
                    {
                        $result = 0;
                        return $result;
                    }
                }
            }
        }
    }

    /**
     * Updates remitter account.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $transfer account transfer parameters
     * @return fault or nothing
     */
    function update_remitter($secId,$lang,$transfer)
    {
        global $girokonto;
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if((!($us_role & utrl_bankmanager || $us_role & utrl_customer))
                && (!($this->standingOrder) && ($us_role & utrl_technicaluser)))
        {
            return getFault("2008");
        }

        $account_transfer_db = &new DBAccess;

        $remitteraccountid = $transfer["remitteraccountid"];
        $remitterbankid = $transfer["remitterbankid"];
        $amount =$transfer["amount"];


        $select_account = "SELECT accountbalance "
                ."FROM ".$cf_db_prefix."account WHERE accountid = '".$remitteraccountid."' "
                ."AND bankid = '".$remitterbankid."'";

        $account_transfer_db->query($select_account);
        if($account_transfer_db->errno != 0)
        {
            return $account_transfer_db->getError();
        }

        // compute new accountbalance
        $account_transfer_db->next_record();
        $rec_accountbalance = $account_transfer_db->Record['accountbalance'];
        $rec_accountbalance = $rec_accountbalance - $amount;

        // make a transfer
        $update_account = "UPDATE ".$cf_db_prefix."account "
                ."SET accountbalance ='".$rec_accountbalance."' "
                ."WHERE accountid = '".$remitteraccountid."' "
                ."AND bankid = '".$remitterbankid."'";

        $account_transfer_db->query($update_account);
        if($account_transfer_db->errno != 0)
        {
            return $account_transfer_db->getError();
        }
    }

    /**
     * Updates transfer table to document the amount of money transfer.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $transfer account transfer parameters
     * @return fault or transfer id
     */
    function transfer($secId,$lang,$transfer)
    {
        global $girokonto;
        $cf_lang = $lang;
        global $cf_db_prefix;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if((!($us_role & utrl_bankmanager || $us_role & utrl_customer))
                && (!($this->standingOrder) && ($us_role & utrl_technicaluser)))
        {
            return getFault("2008");
        }
        $account_transfer_db = &new DBAccess;

        $remitteraccountid = $transfer["remitteraccountid"];
        $remitterbankid = $transfer["remitterbankid"];
        $accountcurrency = $transfer["accountcurrency"];
        $amount =$transfer["amount"];
        $remitteraccountowner = $transfer["remitteraccountowner"];
        $purpose = $transfer["purpose"];
        $payeeaccountid = $transfer["payeeaccountid"];
        $payeeaccountowner = $transfer["payeeaccountowner"];
        $payeebankid = $transfer["payeebankid"];

        $acttime = time();
        $insert_transfer = "INSERT INTO ".$cf_db_prefix."transfer "
                ."(date,remitterbankid,remitteraccountid,payeebankid,payeeaccountid,"
                ."customerid,purpose,currencyid,amount,payeename) "
                ."VALUES('".$acttime."','".$remitterbankid."','".$remitteraccountid."','"
                .$payeebankid."',"."'".$payeeaccountid."','".$remitteraccountowner."','".$purpose."',"
                ."'".$accountcurrency."','".$amount."','".$payeeaccountowner."')";

        $account_transfer_db->query($insert_transfer);
        if($account_transfer_db->errno != 0)
        {
            return $account_transfer_db->getError();
        }
        else
        {
            // this function is used because the mysql function
            // "max" produces an error
            $transferid = mysql_insert_id();
            return $transferid;
        }
    }

    /**
     * Rolls back the transfer database changes if an error occurred during transfer.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $transfer account transfer parameters
     * @return fault or nothing
     */
    function transfer_rollback($secId,$lang,$account)
    {
        global $girokonto;
        $cf_lang = $lang;
        global $cf_db_prefix;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if((!($us_role & utrl_bankmanager || $us_role & utrl_customer))
                && (!($this->standingOrder) && ($us_role & utrl_technicaluser)))
        {
            return getFault("2008");
        }

        $account_transfer_db = &new DBAccess;

        $remitteraccountid = $account["accountid"];
        $remitterbankid = $account["bankid"];
        $transferid = $account["transferid"];
        $amount =$account["amount"];

        // correct remitter account
        $select_account = "SELECT accountbalance "
                ."FROM ".$cf_db_prefix."account "
                ."WHERE accountid='".$remitteraccountid."' "
                ."AND bankid='".$remitterbankid."'";
        $account_transfer_db->query($select_account);

        if($account_transfer_db->errno != 0)
        {
            return $account_transfer_db->getError();
        }

        // compute new accountbalance
        $account_transfer_db->next_record();
        $rec_accountbalance =
        $account_transfer_db->Record['accountbalance'];
        $rec_accountbalance = $rec_accountbalance + $amount;

        // make a transfer
        $update_account = "UPDATE ".$cf_db_prefix."account "
                ."SET accountbalance ='".$rec_accountbalance."' "
                ."WHERE accountid = '".$remitteraccountid."' "
                ."AND bankid = '".$remitterbankid."'";

        $account_transfer_db->query($update_account);

        if($account_transfer_db->errno != 0)
        {
	  return $account_transfer_db->getError();
        }

        // delete transfer from transfertable
        $delete_transfer = "DELETE FROM ".$cf_db_prefix."transfer "
                ."WHERE transferid = '".$transferid."'";

        $account_transfer_db->query($delete_transfer);
        if($account_transfer_db->errno != 0)
        {
	  return $account_transfer_db->getError();
        }
    }

        


    /**
     * Transfers a amount of money from one account to another.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $transfer account transfer parameters
     * @return fault or transfer id
     */
    function transfer_process($secId,$lang,$transfer)
    {
        global $girokonto;
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if((!($us_role & utrl_bankmanager || $us_role & utrl_customer))
                && (!($this->standingOrder) && ($us_role & utrl_technicaluser)))
        {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return getFault("2008");
        }

        $account_transfer_db = &new DBAccess;

        // check for transfer to same account
        if($transfer['remitterbankid'] == $transfer['payeebankid'] &&
                $transfer['remitteraccountid'] == $transfer['payeeaccountid']) {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return getFault("2078");
        }
                        
        //check data of transmitter
        $result = $this->transfer_remitter($secId,$lang,$transfer);

        // error handling
        if(isFault($result))
        {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return $result;
        }

        if($result != 0)
        {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return getFault("2034");
        }
        
        // insert transfer information in transfer table
        $result = $this->transfer($secId,$lang,$transfer);
        $transferid = $result;

        // error handling
        if(isFault($result))
        {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return getFault($result);
        }

        // change values in remitter account
        $result = $this->update_remitter($secId,$lang,$transfer);

        // error handling
        if(isFault($result))
        {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return getFault($result);
        }

        // change values in remittee account
        if ($transfer['payeebankid']<10000)
	  {
	    $result = $this->transfer_payee($secId,$lang,$transfer);
	  }
	else
	  {
	    $result = $this->transfer_payee_international($secId,$lang,$transfer);
	  }
        // error handling
        if($result != 0)
        {
	  
            $result_error = $result;
            // delete record in transfer table
            // reset accountbalance in remitter account
            $account_transfer =  array(
                    "accountid" => $transfer["remitteraccountid"],
                    "bankid" => $transfer["remitterbankid"],
                    "amount" => $transfer["amount"],
                    "transferid" => $transferid);
            $result = $this->transfer_rollback($secId,$lang,$account_transfer);
            
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return getFault($result_error);
        }
        else
        {
            $this->standingOrder = false;
            $this->chargeOrder = false;
            return $transferid;
        }
    }

    /**
     * Produces new money for state economic.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountparams parameters of increase of central bank amount
     * @return fault or transfer id of change
     */
    function account_create_money($secId,$lang,$accountparams)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        // check security id
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        // check role
        if(!($us_role & utrl_centralbankmanager))
        {
            return getFault("2008");
        }

        // check user input
        $result = $this->check_centralaccount($lang,$accountparams);
        if(isFault($result))
        {
            return $result;
        }

        // get bank parameters from user interface
        $account_createmoney_db = &new DBAccess;

        $centralbankid = $accountparams["centralbankid"];
        $centralaccountid = $accountparams["centralaccountid"];
        $centralamount = $accountparams["centralamount"];
        $centralamountinc = $accountparams["centralamountinc"];

        // check bankid and accountid (valid) and kind of account = state
        $check_bankid = "SELECT bankid,accountid,kindofaccount,accountowner,accountbalance,accountowner "
                ."FROM ".$cf_db_prefix."account "
                ."WHERE bankid = '".$centralbankid."' "
                ."AND accountid = '".$centralaccountid."' "
                ."AND kindofaccount = '1'";

        $account_createmoney_db->query($check_bankid);
        if($account_createmoney_db->errno != 0)
        {
            return $account_createmoney_db->getError($cf_lang);
        }
        if($account_createmoney_db->num_rows()    == "0")
        {
            $result = 2016;
            return getFault($result);
        }

        // check accountid of state
        $account_createmoney_db->next_record();
        $accountowner = $account_createmoney_db->Record['accountowner'];
        $accountbalance = $account_createmoney_db->Record['accountbalance'];
        $check_accountid = "SELECT kindofcustomer,customerid "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid = '".$accountowner."' "
                ."AND kindofcustomer = '5'";

        $account_createmoney_db->query($check_accountid);
        if($account_createmoney_db->errno != 0)
        {
            return $account_createmoney_db->getError($cf_lang);
        }
        if($account_createmoney_db->num_rows()    == "0")
        {
            $result    = 2066;
            return getFault($result);
        }

        // check amount
        if($centralamountinc < 0.00)
        {
            return getFault("2067");
        }

        // update central account
        $account_createmoney_db->next_record();
        $accountbalance = $accountbalance + $centralamountinc;

        $update_account = "UPDATE ".$cf_db_prefix."account "
                ."SET accountbalance ='".$accountbalance."' "
                ."WHERE accountid = '".$centralaccountid."' "
                ."AND bankid = '".$centralbankid."'";

        $account_createmoney_db->query($update_account);
        if($account_createmoney_db->errno != 0)
        {
            return $account_createmoney_db->getError();
        }

        // insert ctreate money in transfer table
        $acttime = time();
        $insert_transfer = "INSERT INTO ".$cf_db_prefix."transfer "
                ."(date,payeebankid,payeeaccountid,customerid,purpose,amount) "
                ."VALUES(".$acttime."','".$centralbankid."','".$centralaccountid."', "
                ."'".$us_id."','Guthaben erhht','".$centralamountinc."')";

        $account_createmoney_db->query($insert_transfer);
        if($account_createmoney_db->errno != 0)
        {
            return $account_createmoney_db->getError();
        }
        else
        {
            // this function is used because the mysql function
            // "max" produces an error
            $transferid = mysql_insert_id();
            return $transferid;
        }
    }

    /**
     * Eliminates money for state economic.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountparams parameters of decrease of central bank amount
     * @return fault or transfer id of change
     */
    function account_eliminate_money($secId,$lang,$accountparams)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        // check security id
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        // check role
        if(!($us_role & utrl_centralbankmanager))
        {
            return getFault("2008");
        }

        // check user input
        $result = $this->check_centralaccount($lang,$accountparams);
        if(isFault($result))
        {
            return $result;
        }

        // get bank parameters from user interface
        $account_eliminatemoney_db = &new DBAccess;

        $centralbankid = $accountparams["centralbankid"];
        $centralaccountid = $accountparams["centralaccountid"];
        $centralamount = $accountparams["centralamount"];
        $centralamountdel = $accountparams["centralamountdel"];

        // check bankid and accountid (valid) and kind of account = state
        $check_bankid = "SELECT bankid,accountid,kindofaccount,accountbalance,accountowner "
                ."FROM ".$cf_db_prefix."account "
                ."WHERE bankid = '".$centralbankid."' "
                ."AND accountid = '".$centralaccountid."' "
                ."AND kindofaccount = '1'";

        $account_eliminatemoney_db->query($check_bankid);
        if($account_eliminatemoney_db->errno != 0)
        {
            return $account_eliminatemoney_db->getError($cf_lang);
        }
        if($account_eliminatemoney_db->num_rows()    == "0")
        {
            return getFault(2016);
        }

        // check accountid of state
        $account_eliminatemoney_db->next_record();
        $accountowner = $account_eliminatemoney_db->Record['accountowner'];
        $accountbalance = $account_eliminatemoney_db->Record['accountbalance'];
        $check_accountid = "SELECT kindofcustomer,customerid "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid = '".$accountowner."' "
                ."AND kindofcustomer = '5'";

        $account_eliminatemoney_db->query($check_accountid);
        if($account_eliminatemoney_db->errno != 0)
        {
            return $account_eliminatemoney_db->getError($cf_lang);
        }
        if($account_eliminatemoney_db->num_rows()    == "0")
        {
            return getFault(2066);
        }

        // check amount
        if($centralamountdel < 0.00)
        {
            return getFault("2067");
        }

        // update central account
        $account_eliminatemoney_db->next_record();
        $accountbalance = $accountbalance - $centralamountdel;

        if($accountbalance < 0.0)
        {
            return getFault("2068");
        }

        $update_account = "UPDATE ".$cf_db_prefix."account "
                ."SET accountbalance ='".$accountbalance."' "
                ."WHERE accountid = '".$centralaccountid."' "
                ."AND bankid = '".$centralbankid."'";

        $account_eliminatemoney_db->query($update_account);
        if($account_eliminatemoney_db->errno != 0)
        {
            return $account_createmoney_db->getError();
        }

        // insert ctreate money in transfer table
        $acttime = time();
        $insert_transfer = "INSERT INTO ".$cf_db_prefix."transfer "
                    ."(date,remitterbankid,remitteraccountid,customerid,"
                    ."purpose,amount) "
                    ."VALUES("
                    ."'".$acttime."',"
                    ."'".$centralbankid."',"
                    ."'".$centralaccountid."',"
                    ."'".$us_id."',"
                    ."'Guthaben reduziert',"
                    ."'".$centralamountdel."')";

        $account_eliminatemoney_db->query($insert_transfer);
        if($account_eliminatemoney_db->errno != 0)
        {
            return $account_eliminatemoney_db->getError();
        }
        else
        {
            // this function is used because the mysql function
            // "max" produces an error
            $transferid = mysql_insert_id();
            return $transferid;
        }
    }

    /**
     * Gets a list of all standing orders of one account.
     *
     * @param $secId security id from login
     * @param $lang language
     * @param $accountid acount for which the standing orders are requested
     * @param $bankid id of bank
     * @param $deleted if set to true also deleted standing orders will be read
     * @return array of array of standing order values
     *
     * @see StandingOrder
     */
    function standingorder_list($secId, $lang, $accountid, $bankid, $deleted=false) {
        global $cf_db_prefix;
        
        $select_so = "SELECT * FROM ".$cf_db_prefix."standingorder "
                ."WHERE remitterbankid='".$bankid."' "
                ."AND remitteraccountid='".$accountid."' "
                ."AND deleted='0'";
        $standingorder_db = &new DBAccess;
        $standingorder_db->query($select_so);

        if($standingorder_db->errno != 0)
        {
            return $standingorder_db->getError();
        }
        $i=0;
        while($standingorder_db->next_record()) {
            if($deleted || $standingorders[$i]['deleted']==0) {
                $standingorders[$i]['soid'] = $standingorder_db->Record["soid"];
                $standingorders[$i]['remitterbankid'] = $standingorder_db->Record["remitterbankid"];
                $standingorders[$i]['remitteraccountid'] = $standingorder_db->Record["remitteraccountid"];
                $standingorders[$i]['payeebankid'] = $standingorder_db->Record["payeebankid"];
                $standingorders[$i]['payeeaccountid'] = $standingorder_db->Record["payeeaccountid"];
                $standingorders[$i]['customerid'] = $standingorder_db->Record["customerid"];
                $standingorders[$i]['purpose'] = $standingorder_db->Record["purpose"];
                $standingorders[$i]['currencyid'] = $standingorder_db->Record["currencyid"];
                $standingorders[$i]['amount'] = $standingorder_db->Record["amount"];
                $standingorders[$i]['payeename'] = $standingorder_db->Record["payeename"];
                $standingorders[$i]['period'] = $standingorder_db->Record["period"];
                $standingorders[$i]['time'] = $standingorder_db->Record["time"];
                $standingorders[$i]['lastexecuted'] = $standingorder_db->Record["lastexecuted"];
            }
            $i++;
        }
        return $standingorders;
    }
    
    /**
     * Sets the standing order flag.
     *
     * Use it only before call to transfer_process if executing a standing order.
     * Only locally available!
     */
    function setStandingOrder() {
        $this->standingOrder = true;
    }
    
    /**
     * Sets the charge order flag.
     *
     * Use it only before call to transfer_process if executing a standing order.
     * Only locally available!
     */
    function setChargeOrder() {
        $this->chargeOrder = true;
    }
    

    /**
     * Charge fees for account usage.
     *
     * @access protected
     * @param $accountAttrs attributes of account
     * @param $bankAttrs attributes of bank
     */
    function charge_fees($secId, $lang, $accountAttrs, $bankAttrs) {
        global $cf_db_prefix;

        // get current date
        $dateArray = getdate();
        $currMonth = $dateArray['mon'];
        $currYear = $dateArray['year'];
        $currDate = $currYear*100 + $currMonth;
        
        // get charge date
        $dateArray = getdate($accountAttrs['lastfeedate']);
        $chargeMonth = $dateArray['mon'];
        $chargeYear = $dateArray['year'];
        $chargeDate = $chargeYear*100 + $chargeMonth;
        
        // for all months
        while($chargeDate < $currDate) {
            
            // initalize purpose text
            $purpose = '';
                    
            // calculate time period for charge        
            $fromdate = mktime(0,0,0,$chargeMonth,1,$chargeYear);
            $todate =  mktime(0,0,0,$chargeMonth+1,0,$chargeYear);
             
            // get account opening fee
            if($bankAttrs['accountstartfee'] != 0) {
                $openDateArray = getdate($accountAttrs['lastfeedate']);
                $openDate = $openDateArray['year'] * 100 + $openDateArray['mon'];
                if($openDate == $chargeDate) {
                    $fee += $bankAttrs['accountstartfee'];
                    $purpose .= 'Er&ouml;ffnung='.$bankAttrs['accountstartfee'];
                }
            }

            // get monthly giro charge
            if($bankAttrs['girocharge'] != 0) {
                $fee += $bankAttrs['girocharge'];
                $purpose .= 'Monatsgeb&uuml;hr='.$bankAttrs['accountstartfee'];
            }
            
            // get remittance fee
            if($bankAttrs['remittancefeesame'] != 0
                    || $bankAttrs['remittancefeeother'] != 0
                    || $bankAttrs['remittancefeeinternational'] != 0) {

                $transfers = $account->account_extract($secId,$lang,
                    $accountAttrs['accountid'],$bankid,$fromdate,$todate);
                    
                $noRemittancesSame = 0;
                $noRemittancesOther = 0;
                $noRemittancesInternational = 0;
                foreach($transfers as $transfer) {
                    if($transfer['remitterbankid'] == $accountAttrs['bankid']) {
                        // sort out administrative entries (account balances)
                        if($transfer['remitteracountid'] != $transfer['payeeaccountid'])
                            $noRemittancesSame++;
                    }
                    else {
                        if(isalpha(substr($transfer['payeebankid'],0)))
                            $noRemittancesInternational++;
                        else
                            $noRemittancesOther++;
                    }
                }
                if($noRemittancesSame > 0 && $bankAttrs['remittancefeesame'] != 0) {
                    $fee += $noRemittancesSame * $bankAttrs['remittancefeesame'];
                    $purpose .= " ".$noRemittancesSame."x&Uuml;berweisung";
                }
                if($noRemittancesOther > 0 && $bankAttrs['remittancefeeother'] != 0) {
                    $fee += $noRemittancesOther * $bankAttrs['remittancefeeother'];
                    $purpose .= " ".$noRemittancesOther."x&Uuml;berweisung(andere Bank)";
                }
                if($noRemittancesInternational > 0 && $bankAttrs['remittancefeeinternational'] != 0) {
                    $fee += $noRemittancesInternational * $bankAttrs['remittancefeeinternational'];
                    $purpose .= " ".$noRemittancesInternational."x&Uuml;berweisung(anderer Staat)";
                }
                
                $transfers = 0;
            }
            
            // standing order fees
            if($bankAttrs['standingorderfirstfee'] != 0
                    || $bankAttrs['standingorderendfee'] != 0
                    || $bankAttrs['standingorderchangefee'] != 0) {
                        
                // get list of standing orders
                $standingorders = standingorder_list($secId, $lang,
                        $accountAttrs['accountid'], $accountAttrs['bankid'], true);
                $noCreated = 0;
                $noDeleted = 0;
                $noChanged = 0;
                foreach($standingorders as $standingorder) {
                    $dateArray = $standingorder['createdate'];
                    $createdate = $dateArray['year']*100 + $dateArray['mon'];
                    $dateArray = $standingorder['deletedate'];
                    $deletedate = $dateArray['year']*100 + $dateArray['mon'];
                    $dateArray = $standingorder['changedate'];
                    $changeDate = $dateArray['year']*100 + $dateArray['mon'];
                    if($createdate == $chargeDate)
                        $noCreated++;
                    if($deletedate == $chargeDate)
                        $noDeleted++; 
                    if($changeDate == $chargeDate)
                        $noChanged++; 
                }
                if($noCreated != 0
                        && $bankAttrs['standingorderfirstfee'] != 0) {
                    $fee += $noCreated * $bankAttrs['standingorderfirstfee'];
                    $purpose .= " ".$noCreated."xDauerauftrag angelegt";
                }
                if($noDeleted != 0
                        && $bankAttrs['standingorderendfee'] != 0) {
                    $fee += $noDeleted * $bankAttrs['standingorderendfee'];
                    $purpose .= " ".$noCreated."xDauerauftrag gel&ouml;scht";
                }
                if($noCreated != 0
                        && $bankAttrs['standingorderchangefee'] != 0) {
                    $fee += $noCreated * $bankAttrs['standingorderchangefee'];
                    $purpose .= " ".$noCreated."xDauerauftrag gendert";
                }
            }
                                
            // transfer money from customer account to bank account
            $transfer['remitterbankid'] = $accountAttrs['bankid'];
            $transfer['remitteraccountid'] = $accountAttrs['accountid'];
            $transfer['payeebankid'] = $accountAttrs['bankid'];
            $transfer['payeeaccountid'] = $accountAttrs['feeaccount'];
            $transfer["amount"] = $fee;
            $transfer["purpose"] = $purpose;
            $transfer["accountowner"] = $bankAttrs['bankmanagerid'];
            $transfer["payeeaccountowner"] = "";
            $transfer["accountcurrency"] = $accountAttrs['accountcurrency'];
            $this->setChargeOrder();
            $result = transfer_process($secId,$lang,$transfer);

            // compute new charge date
            if(! isFault($result)) {
                $chargeTime = mktime(0,0,0,$chargeMonth+1,1,$chargeYear);
                $dateArray = gettime($chargeTime);
                $chargeMonth = $dateArray['mon'];
                $chargeYear = $dateArray['year'];
                $chargeDate = $chargeyear*100 + $chargeMonth;
            }
            
            // update last fee date
            if(!isFault($ret)) {
                $sql = "UPDATE ".$cf_db_prefix."account "
                        ."SET lastfeedate='".$chargedate."',"
                        ."WHERE accountid='".$accountAttrs['accountid']."' "
                            ."AND bankid='".$accountAttrs['bankid']."'";

                $db->query($sql);
        
                if($db->errno != 0) {
                    $result = $db->getError();
                 }
             }

             // return result
             return $result;
        }
    }
}
?>