<?php
/**
 * MiNBank business object: bank
 *
 * $Source: /cvsroot/minbank/minba/minba/bo/babo_bank.php,v $
 * $Author: eltres $
 * $Date: 2007/01/22 20:46:47 $
 * $Revision: 1.10 $
 */

require_once("../bagn_config.php");
$appl_old = $appl_path;
require_once("$minsoft_path/user/bo/usbo_security.php");
$appl_path = $appl_old;
require_once("$appl_path/bo/babo_globmeth.php");
require_once("$minsoft_path/utility/utdb_access.php");
require_once("$minsoft_path/utility/utgn_config.php");
require_once("$minsoft_path/utility/utrl_role.php");
require_once("$minsoft_path/utility/utgn_checkit.php");
require_once("$minsoft_path/utility/utxr_client.php");
require_once("$minsoft_path/user/usgn_config.php");

/**
 * Bank
 *
 * Attributes are:
 * <table>
 * <tr><td>bankname</td><td>Name of bank.</td></tr>
 * <tr><td>bankid</td><td>Bank identification code.</td></tr>
 * <tr><td>bankmanagerid</td><td>User id of bankmanager.</td></tr>
 * <tr><td>feeaccount</td><td>Account id as payee for bank fees.</td></tr>
 * <tr><td>accountstartfee</td><td>Fee for creation of an account.</td></tr>
 * <tr><td>accountendfee</td><td>Fee for removal of an account.</td></tr>
 * <tr><td>girocharge</td><td>Monthly charge for giro account.</td></tr>
 * <tr><td>remittancefeesame</td><td>Charge for a remitted transfer in same bank.</td></tr>
 * <tr><td>remittancefeeother</td><td>Charge for a remitted transfer to other bank.</td></tr>
 * <tr><td>remittancefeeinternational</td><td>Charge for a remitted transfer to other nation.</td></tr>
 * <tr><td>standingorderfirstfee</td><td>Fee for creation of a standing order.</td></tr>
 * <tr><td>standingorderendfee</td><td>Fee for removal of a standing order.</td></tr>
 * <tr><td>standingorderchangefee</td><td>Fee for change of a standing order.</td></tr>
 * <tr><td>bankwithdrawalstartfee</td><td>Fee for creation of a bank withdrawal.</td></tr>
 * <tr><td>bankwithdrawalendfee</td><td>Fee for removal of a bank withdrawal.</td></tr>
 * <tr><td>overdraftinterest</td><td>Interest per month for overdrafts.</td></tr>
 * <tr><td>creditfee</td><td>Fee for creation of a credit.</td></tr>
 * <tr><td>creditinterest</td><td>Interest rate per month for credits.</td></tr>
 * <tr><td>overdraftinterest</td><td>Overdraft interest rate per month.</td></tr>
 * <tr><td>timedeposit1name</td><td>Name of first time deposit offer.</td></tr>
 * <tr><td>timedeposit1period</td><td>Period in days that the money is blocked of first time deposit offer.</td></tr>
 * <tr><td>timedeposit1interest</td><td>Interest percentage per month of first time deposit offer.</td></tr>
 * <tr><td>timedeposit2name</td><td>Name of second time deposit offer.</td></tr>
 * <tr><td>timedeposit2period</td><td>Period in days that the money is blocked of second time deposit offer.</td></tr>
 * <tr><td>timedeposit2interest</td><td>Interest percentage per month of second time deposit offer.</td></tr>
 * <tr><td>timedeposit3name</td><td>Name of third time deposit offer.</td></tr>
 * <tr><td>timedeposit3period</td><td>Period in days that the money is blocked of third time deposit offer.</td></tr>
 * <tr><td>timedeposit3interest</td><td>Interest percentage per month of third time deposit offer.</td></tr>
 * <tr><td>timedeposit4name</td><td>Name of fourth time deposit offer.</td></tr>
 * <tr><td>timedeposit4period</td><td>Period in days that the money is blocked of fourth time deposit offer.</td></tr>
 * <tr><td>timedeposit4interest</td><td>Interest percentage per month of fourth time deposit offer.</td></tr>
 * <tr><td>createdate</td><td>date the account has been created</td></tr>
 * <tr><td>deletedate</td><td>date the account has been deleted</td></tr>
 * <tr><td>changedate</td><td>date the account has been deleted</td></tr>
 * <tr><td>deleted</td><td>deletion flag</td></tr>
 * <tr><td>locked</td><td>lock flag</td></tr>
 * </table>
 */
class Bank
{
    // ****************   public class methods ******************************

    /**
     * Lists active customers belonging to a bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bankid bank id
     * @return fault or array of customers
     */
    function customer_list($secId,$lang,$bankid)
    {
        global $cf_db_prefix;

        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if(!($us_role & utrl_bankmanager | $us_role &  utrl_technical))
        {
            return getFault("2008");
        }

        $customer_list_db = &new DBAccess;

        // create customer table
        $select_custlist =
                "SELECT customerid,customername, kindofcustomer "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE bankid = '".$bankid."' "
                    ."AND deleted ='0' "
                ."ORDER BY kindofcustomer,customerid";

        $customer_list_db->query($select_custlist);
        if($customer_list_db->errno != 0)
        {
            return $customer_list_db->getError();
        }
        if($customer_list_db->num_rows() == 0)
        {
            $custlist = NULL;
            return getFault("2013");

        }
        else
        {

            // fill array with customer list
            while ($customer_list_db->next_record())
            {
                $custlist[] = array(
                    "customerid"=>$customer_list_db->Record['customerid'],
                    "customername"=>$customer_list_db->Record['customername'],
                    "kindofcustomer"=>$customer_list_db->Record['kindofcustomer']);
            }
        }
        return  $custlist;
    }

    /**
     * Lists inactive customers belonging to a bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bankid bank id
     * @return fault or array of customers
     */
    function customer_list_inactive($secId,$lang,$bankid)
    {
        global $cf_db_prefix;

        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if(!($us_role & utrl_bankmanager))
        {
            return getFault("2008");
        }

        $customer_list_db = &new DBAccess;

        // create customer table
        $select_custlist = "SELECT  customerid,customername,kindofcustomer "
            ."FROM ".$cf_db_prefix."customer "
            ."WHERE bankid='".$bankid."' "
            ."AND deleted ='99' "
            ."ORDER BY kindofcustomer,customerid";

        $customer_list_db->query($select_custlist);
        if($customer_list_db->errno != 0)
        {
            return $customer_list_db->getError();
        }
        if($customer_list_db->num_rows() == 0)
        {
            $custlist = NULL;
            //return getFault("2013");

        }
        else
        {
            // fill array with customer list
            while ($customer_list_db->next_record())
            {
                $custlist[] = array(
                    "customerid"=>$customer_list_db->Record['customerid'],
                    "customername"=>$customer_list_db->Record['customername'],
                    "kindofcustomer"=>$customer_list_db->Record['kindofcustomer']);
            }
        }
        return  $custlist;
    }

    /**
     * Gets the parameters of a bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bankid bank id
     * @return fault or array of parameters
     */
    function bank_read_params($secId,$lang,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if(!($us_role & (utrl_bankmanager | utrl_centralbankmanager)))
        {
            return getFault("2008");
        }

        $bank_read_db = &new DBAccess;

        // read parameters
        $select_bankparams = "SELECT  * "
            ."FROM ".$cf_db_prefix."bank "
            ."WHERE bankid='".$bankid."'";

        $bank_read_db->query($select_bankparams);
        if($bank_read_db->errno != 0)
        {
            return $bank_read_db->getError();
        }
        if($bank_read_db->num_rows() == 0)
        {
            $bankparams = NULL;
            return getFault("2013");
        }
        else
        {
            // fill array with bank params
            while ($bank_read_db->next_record())
            {
                $bankparams[] = array(
                    "bankname" => $bank_read_db->Record['bankname'],
                    "bankid" => $bank_read_db->Record['bankid'],
                    "bankaccountid" => $bank_read_db->Record['bankaccountid'],
                    "accountstartfee" => $bank_read_db->Record['accountstartfee'],
                    "accountendfee" => $bank_read_db->Record['accountendfee'],
                    "girocharge" => $bank_read_db->Record['girocharge'],
                    "remittancefee" => $bank_read_db->Record['remittancefee'],
                    "standingorderfirstfee" => $bank_read_db->Record['standingorderfirstfee'],
                    "standingorderendfee" =>  $bank_read_db->Record['standingorderendfee'],
                    "bankwithdrawalstartfee" => $bank_read_db->Record['bankwithdrawalstartfee'],
                    "bankwithdrawalendfee" => $bank_read_db->Record['bankwithdrawalendfee'],
                    "overdraftinterest" => $bank_read_db->Record['overdraftinterest'],
                    "creditfee" => $bank_read_db->Record['creditfee'],
                    "creditinterest" => $bank_read_db->Record['creditinterest'],
                    "overdraftinterest" => $bank_read_db->Record['overdraftinterest'],
                    "creditinterest" => $bank_read_db->Record['creditinterest'],
                    "bankmanagerid" => $bank_read_db->Record['bankmanagerid'],
                    "createdate" => $bank_read_db->Record['createdate'],
                    "deletedate" => $bank_read_db->Record['deletedate'],
                    "changedate" => $bank_read_db->Record['changedate'],
                    "lockdate" => $bank_read_db->Record['lockdate']);
                    
                $this->timedeposits_read(&$bankparams);
            }
        }
        return  $bankparams;
    }

    /**
     * Updates the parameters of a bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bankparams bank parameters
     * @return fault or nothing
     */
    function bank_update_params($secId,$lang,$bank)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }

        if(!($us_role & utrl_bankmanager))
        {
            return getFault("2008");
        }

        // check user input
        $result = $this->check_bank_params($lang,$bank);
         if(isFault($result))
        {
            return $result;
        }

        // update bank parameters
        $bank_update_db = &new DBAccess;

        $update_bank_params = "UPDATE ".$cf_db_prefix."bank "
                ."SET bankname ='".$bank['bankname']."',"
                ."bankaccountid = '".$bank['bankaccountid']."',"
                ."accountstartfee = '".$bank['accountstartfee']."',"
                ."accountendfee = '".$bank['accountendfee']."',"
                ."girocharge = '".$bank['girocharge']."',"
                ."remittancefee ='".$bank['remittancefee']."',"
                ."standingorderfirstfee ='".$bank['standingorderfirstfee']."',"
                ."standingorderendfee ='".$bank['standingorderendfee']."',"
                ."bankwithdrawalstartfee ='".$bank['bankwithdrawalstartfee']."',"
                ."bankwithdrawalendfee   ='".$bank['bankwithdrawalendfee']."',"
                ."overdraftinterest  ='".$bank['overdraftinterest']."',"
                ."creditfee  ='".$bank['creditfee']."',"
                ."creditinterest  ='".$bank['creditinterest']."',"
                ."overdraftinterest  ='".$bank['overdraftinterest']."',"
                ."changedate  ='".time()."' "
                ."WHERE bankid = '".$bank['bankid']."'";

        $bank_update_db->query($update_bank_params);
        if($bank_update_db->errno != 0)
        {
            return $bank_update_db->getError($cf_lang);
        }

        $result = $this->timedeposits_update($bank);
        if(isFault($result))
        {
            return $result;
        }
    }

    /**
     * Creates a bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bank bank id
     * @return fault or nothing
     */
    function bank_create($secId,$lang,$bank)
    {
        global $cf_db_prefix;
        global $uscf_db_prefix;

        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_centralbankmanager))
        {
            return getFault("2008");
        }

        $result = $this->check_bank($lang,$bank);
        if(isFault($result))
        {
            return $result;
        }

        $bank_create = &new DBAccess;

        $bankname = $bank["bankname"];
        $bankid = $bank["bankid"];
        $bankmanagerid = $bank["bankmanagerid"];

        //check if userid in user table
        $check_bankmanagerid = "SELECT userid,role "
                ."FROM ".$uscf_db_prefix."user "
                ."WHERE userid='".$bankmanagerid."'";

        $bank_create->query($check_bankmanagerid);
        if($bank_create->errno != 0)
        {
            return $bank_create->getError($cf_lang);
        }
        if($bank_create->num_rows() == 0)
        {
            return getFault("2055");
        }
        else
        {
            // check role of user
            $bank_create->next_record();
            $bk_role = $bank_create->Record['role'];
            if(!($bk_role & utrl_bankmanager))
            {
                return getFault("2056");
            }

            // check bankid
            $check_blkz = "SELECT bankid "
                    ."FROM ".$cf_db_prefix."bank "
                    ."WHERE bankid = '".$bankid."'";

            $bank_create->query($check_blkz);
            if($bank_create->errno != 0)
            {

                return $bank_create->getError($cf_lang);
            }

            if($bank_create->num_rows() != 0)
            {
                return getFault("2060");
            }

            // insert into bank table new bank
            $insert_bank = "INSERT INTO ".$cf_db_prefix."bank "
                ."(bankname,bankid,bankmanagerid,createdate) "
                ."VALUES ('".$bankname."','".$bankid."','".$bankmanagerid."','".time()."')";

            $bank_create->query($insert_bank);
            if($bank_create->errno != 0)
            {
                return $bank_create->getError($cf_lang);
            }
            $insert_bank = "INSERT INTO ".$cf_db_prefix."customer "
                ."VALUES ('".$bankmanagerid."','1','','','','Bankmanager','','$bankid','','0')";

            $bank_create->query($insert_bank);
            if($bank_create->errno != 0)
            {
                return $bank_create->getError($cf_lang);
            }
        }
    }

    /**
     * Updates a bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bank bank id
     * @return fault or nothing
     */
    function bank_update($secId,$lang,$bank)
    {
        global $cf_db_prefix;
        global $uscf_db_prefix;

        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_centralbankmanager))
        {
            return getFault("2008");
        }

        $result = $this->check_bank($lang,$bank);
        if(isFault($result))
        {
            return $result;
        }

        $bank_update = &new DBAccess;

        $bankname = $bank["bankname"];
        $bankid = $bank["bankid"];
        $bankmanagerid = $bank["bankmanagerid"];

        //check if userid in user table
        $check_bankmanagerid = "SELECT userid,role "
                ."FROM ".$uscf_db_prefix."user "
                ."WHERE userid = '".$bankmanagerid."'";

        $bank_update->query($check_bankmanagerid);
        if($bank_update->errno != 0)
        {
            return $bank_update->getError($cf_lang);
        }
        if($bank_update->num_rows() == 0)
        {
            return getFault("2055");
        }
        else
        {
            $bank_update->next_record();
            $customer_role = $bank_update->Record['role'];

            if(!($customer_role & utrl_bankmanager))
            {
                return getFault("2069");
            }
            // update bank parameters
            $update_bank = "UPDATE ".$cf_db_prefix."bank "
                    ."SET bankname='".$bankname."',"
                        ."bankmanagerid='".$bankmanagerid."',"
                        ."changedate='".time()."' "
                    ."WHERE bankid='".$bankid."'";

            $bank_update->query($update_bank);
            if($bank_update->errno != 0)
            {
                return $bank_update->getError($cf_lang);
            }
        }
    }

    /**
     * Deletes a bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bankid bank id
     * @return fault or nothing
     */
    function bank_delete($secId,$lang,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & utrl_centralbankmanager))
        {
            return getFault("2008");
        }

        $bank_delete = &new DBAccess;

        //check if bankid in bank table
        $check_bankid = "SELECT bankmanagerid "
                ."FROM ".$cf_db_prefix."bank "
                ."WHERE bankid='".$bankid."'";

        $bank_delete->query($check_bankid);
        if($bank_delete->errno != 0)
        {
            return $bank_delete->getError($cf_lang);
        }
        if($bank_delete->num_rows() == 0)
        {
            return getFault("2056");
        }
        else
        {
            // check if bank has accounts with amount greater zero
            $check_customer = "SELECT customerid "
                    ."FROM ".$cf_db_prefix."customer "
                    ."WHERE bankid='".$bankid."' "
                    ."AND deleted='0'";

            $bank_delete->query($check_customer);
            if($bank_delete->errno != 0)
            {
                return $bank_delete->getError($cf_lang);
            }
            if($bank_delete->num_rows() != 0)
            {
                return getFault("2057");
            }

            // delete bank from bank table
            $delete_bank = "UPDATE {$cf_db_prefix}bank "
                    ."SET deleted='1',"
                            ."deletiondate='".time()."'"
                        ."WHERE bankid = '".$bankid."'";

            $bank_delete->query($delete_bank);
            if($bank_delete->errno != 0)
            {
                return $bank_delete->getError($cf_lang);
            }
        }
    }

    /**
     * Charge fees and interests of all accounts for all months it is not done.
     * This can be done by the technical user or the bank manager. Work is done on
     * a per account basis, so if time is not sufficient for all accounts,
     * it can be called several times. The money transfer is done directly, because
     * it is always inside one bank.
     *
     * @param $secId security id
     * @param $lang language for messages
     * @param $bankid bank identification code of bank to be handled
     * @return fault or nothing
     */
    function chargeFees($secId,$lang,$bankid) {
        global $cf_db_prefix;

        $require_once("babo_account.php");
        
        // make security checks
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult)) {
            return $secResult;
        }
        else {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];
        }
        if(!($us_role & (utrl_centralbankmanager | utrl_technical))) {
            return getFault("2008");
        }

        // get bank parameters
        $db = &new DBAccess;

        $sql = "SELECT * "
                ."FROM ".$cf_db_prefix."bank "
                ."WHERE bankid='".$bankid."'";
        $db->query($sql);

        if($db->errno != 0) {
            return $db->getError();
        }
        
        if($db->num_rows() == 0 ) {
            return getFault("2013");
        }
        
        $bankAttrs = $db->Record;
        
        // get list of customers
        $customers = $this->customer_list($secId,$lang,$bankid);
        
        if(isFault($customers)) {
            return $customers;
        }

        // for all customers
        $customer = new Customer;
        
        foreach($customers as $customerAttrs) {            
            // get list of accounts
            $accounts = $customer->account_list($secId,$lang,$customerAttrs['customerid'],$bankid);
            
            if(isFault($accounts)) {
                return $accounts;
            }
            
            // for all accounts
            $account = new Account;
            foreach($accounts as $accountAttrs) {
                // charge fee of account
                $account = new Account();
                $account->chargeFees($secId,$lang,$accountAttrs,$bankAttrs);
            }
        }
    }
    
    //******************************* private class methods ************************

    /**
     * Gets a word by a name depending on the language.
     *
     * @access private
     * @param $lang language of word
     * @param $name name of word
     */
    function getKeyword($lang, $name) {
        global $minsoft_path;
        require("$minsoft_path/minba/msg/keywords_".$lang.".txt");
        return $keywords[$name];
    }

    /**
     * Checks the parameters of a bank.
     *
     * @access private
     * @param $bankparams bank parameters
     * @return fault or nothing
     */
    function check_bank_params($lang,$bankparams)
    {
        $check = array("bankname"=>2, "bankaccountid"=>6, "accountstartfee"=>3, "accountendfee"=>3,"girocharge"=>3,
                "remittancefee"=>3,"standingorderfirstfee"=>3, "standingorderendfee"=>3,
                "bankwithdrawalstartfee"=>3, "bankwithdrawalendfee"=>3,
                "overdraftinterest"=>3,"creditfee"=>3,"creditinterest"=>5,"overdraftinterest"=>5,
                "timedeposit1name"=>1,"timedeposit1period"=>4,"timedeposit1interest"=>5,
                "timedeposit2name"=>1,"timedeposit2period"=>4,"timedeposit2interest"=>5,
                "timedeposit3name"=>1,"timedeposit3period"=>4,"timedeposit3interest"=>5,
                "timedeposit4name"=>1,"timedeposit4period"=>4,"timedeposit4interest"=>5,
        );
        foreach ($check as $checkname => $checkitem)
        {
            $item = $bankparams["$checkname"];
            switch($checkitem)
            {
                case 1: // check alphameric
                    if($item != '') {
                        if(!checkit::isAlpha($item,TRUE))
                        {
                            return getFault("2004",$this->getKeyword($lang,"$checkname"));
                        }
                    }
                    break;
                case 2: // check alphanumeric
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                         return getFault("2005",$this->getKeyword($lang,"$checkname"));
                    }
                    break;
                case 3: // check numeric (may include decimal places)
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$this->getKeyword($lang,"$checkname"));
                    }
                    break;
                case 4: // check for positive number (may include decimal places), maybe empty
                    if($item != "") {
                        if(! is_numeric($item))
                        {
                            return getFault("2006",$this->getKeyword($lang,"$checkname"));
                        }
                        if($item < 0)
                        {
                            return getFault("2079",$this->getKeyword($lang,"$checkname"));
                        }
                    }
                    break;
                case 5: // check for interest rate (not higher than 99%), maybe empty
                    if($item != "") {
                        $item =$bankparams["$checkname"];
                        if(! is_numeric($item))
                        {
                            return getFault("2006",$this->getKeyword($lang,"$checkname"));
                        }
                        if($item < 0 || $item > 99)
                        {
                            return getFault("2080",$this->getKeyword($lang,"$checkname"));
                        }
                    }
                    break;
                case 6: // check for whole number
                    if($item == "" || ! is_numeric($item) || intval(0+$item )!= $item || $item<0) {
                            return getFault("2006",$this->getKeyword($lang,"$checkname"));
                    }
                    break;
                default:
                    return getFault("2007");
                    break;
            }
        }
    }

    /**
     * Checks the input of the central bank manager.
     *
     * @access private
     * @param $bank bank attributes
     * @return fault or nothing
     */
    function check_bank($lang,$bank)
    {
        $check = array("bankname"=>2, "bankid"=>3, "bankmanagerid"=>2);

        foreach ($check as $checkname => $checkitem)
        {
            switch($checkitem)
            {
                case 1:
                    $item = $bank["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$this->getKeyword($lang,"$checkname"));
                        break;
                    }
                    if(!checkit::isAlpha($item,TRUE))
                    {
                        return getFault("2004",$this->getKeyword($lang,"$checkname"));
                        break;
                    }
                    break;
                case 2:
                    $item = $bank["$checkname"];
                     if($item == "")
                    {
                        return getFault("2052",$this->getKeyword($lang,"$checkname"));
                        break;
                    }
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                            return getFault("2005",$dname["$checkname"]);
                            break;
                    }
                    break;
                case 3:
                    $item =$bank["$checkname"];
                     if($item == "")
                    {
                        return getFault("2052",$this->getKeyword($lang,"$checkname"));
                        break;
                    }
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$this->getKeyword($lang,"$checkname"));
                        break;
                    }
                    break;
                default:
                    return getFault("2007");
                    break;
            }
        }
    }

    /**
     * Read time deposits of bank into attributes.
     *
     * @param $bank bank attributes
     * @return nothing or fault
     */    
    function timedeposits_read($bank) {
        global $cf_db_prefix;
        // read interests of bank from table
        $db = &new DBAccess;

        $sql = "SELECT * "
                ."FROM ".$cf_db_prefix."timedeposits "
                ."WHERE bankid='".$bank[0]['bankid']."' "
                ."ORDER BY sequence";
        $db->query($sql);
        if($db->errno != 0)
        {
            return $db->getError();
        }

        // fill bank parameters with interests
        $i=1;
        while($db->next_record())
        {
            $bank[0]['timedeposit'.$i.'name'] = $db->Record['name'];
            $bank[0]['timedeposit'.$i.'period'] = $db->Record['period'];
            $bank[0]['timedeposit'.$i.'interest'] = $db->Record['interestrate'];
            $i++;
        }
        
        // fill up unused timedeposits
        for($j=$i; $j<=4; $j++) {
            $bank[0]['timedeposit'.$j.'name'] = $db->Record['name'];
            $bank[0]['timedeposit'.$j.'period'] = $db->Record['period'];
            $bank[0]['timedeposit'.$j.'interest'] = $db->Record['interestrate'];
        }
    }
    
    /**
     * Update time deposits of bank.
     *
     * @param $bank bank attributes
     * @return nothing or fault
     */
    function timedeposits_update($bank) {
        global $cf_db_prefix;

        // delete old interests of bank from table
        $db = &new DBAccess;

        $sql = "DELETE FROM ".$cf_db_prefix."timedeposits "
                ."WHERE bankid='".$bank['bankid']."'";
                
        $db->query($sql);
        if($db->errno != 0)
        {
            return $db->getError();
        }
        
        $seq=0;
        for($i=1; $i<=4; $i++) {
            $name = $bank['timedeposit'.$i.'name'];
            if($name != "") {
                $sql = "INSERT INTO ".$cf_db_prefix."timedeposits "
                    ."(bankid,sequence,name,period,interestrate) "
                    ."VALUES ('".$bank['bankid']."','".$seq."','".$name."',"
                        ."'".$bank['timedeposit'.$i.'period']."',"
                        ."'".$bank['timedeposit'.$i.'interest']."')";

            $db->query($sql);
            if($db->errno != 0)
            {
                return $db->getError();
            }
        
                $seq++;
            }
        }
    }
}
?>