<?php
/**
 * MiNBank business object: central bank
 *
 * $Source: /cvsroot/minbank/minba/minba/bo/babo_centralbank.php,v $
 * $Author: mboller $
 * $Date: 2006/01/05 00:55:27 $
 * $Revision: 1.4 $
 */

require_once("../bagn_config.php");
$appl_old = $appl_path;
require_once("$minsoft_path/user/bo/usbo_security.php");
$appl_path = $appl_old;
require_once("$appl_path/bo/babo_globmeth.php");
require_once("$minsoft_path/utility/utdb_access.php");
require_once("$minsoft_path/utility/utgn_config.php");
require_once("$minsoft_path/utility/utrl_role.php");
require_once("$minsoft_path/utility/utgn_checkit.php");
require_once("$minsoft_path/utility/utxr_client.php");

/**
 * The central bank controls the banks and tries to control
 * the economic amount of money.
 */
class Centralbank
{    
    // ************** public class methods *******************************
    
    /**
     * Gets a list of all banks of a country.
     *
     * @param $secId security id from login
     * @param $lang language for messages
     * @return fault or array of banks
     */
    function getbanklist($secId,$lang)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer || $us_role & utrl_technical)) 
        {
            return getFault(2003);
        }
        else
        {
            $bankma_db = &new DBAccess;
            $select_bankid = 
            "SELECT bankid, bankname,bankmanagerid from {$cf_db_prefix}bank";
            $bankma_db->query($select_bankid);        
            
            if($bankma_db->errno != 0)
            {
                return getFault(50,$user_db->msg);
            }
            else
            { 
                  $count=0;
                // insert values in array
                while ($bankma_db->next_record())
                {
                    $culist[$count]["bankid"] = $bankma_db->Record["bankid"];
                    $culist[$count]["bankname"] = $bankma_db->Record["bankname"];
                    $culist[$count]["bankmanagerid"] = $bankma_db->Record["bankmanagerid"];
                    $count++;
                }
            }
        }
        return  $culist;
    }
        
    /**
     * Checks centralbankmanager/minister of economics
     *
     * @param $secId security id from login
     * @param $lang language for messages
     * @return fault or array of roles
     */ 
    function check_roles($secId,$lang)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $all_roles = NULL;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        $count = 0;
        if($us_role & utrl_mineconomics)
        {
            $all_roles[utrl_mineconomics] = "Wirtschaftsminister";
            $count++;
        }
        if($us_role & utrl_centralbankmanager)
        {
            $all_roles[utrl_centralbankmanager] = "Zentralbankchef";
            $count++;
        }
        if($us_role & utrl_bankmanager)
        {
            $all_roles[utrl_bankmanager] = "Bankdirektor";
            $count++;
        }
        if($us_role & utrl_customer)
        {
            $all_roles[utrl_customer] = "Kunde";
            $count++;
        }
        return $all_roles;        
    }
    
    /**
     * Gets the parameters of the centralbank.
     *
     * @param $secId security id from login
     * @param $lang language for messages
     * @return fault or central bank parameter array
     */
    function centralbank_read_params($secId,$lang)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_centralbankmanager))
        {
            return getFault("2008");
        }
       
        $centralbank_read_db = &new DBAccess;
            
        // read parameters
        $select_centralbankparams = "SELECT  * 
            from {$cf_db_prefix}centralbank";
            
           
        $centralbank_read_db->query($select_centralbankparams);
        if($centralbank_read_db->errno != 0)
        {
            return $centralbank_read_db->getError();
        }
        if($centralbank_read_db->num_rows() == 0)
        {
            $centralbankparams = NULL;
            return getFault("2013");
            
        }
        else
        {   
        
            // fill array with centralbank params
            while ($centralbank_read_db->next_record())
            {
            $centralbankparams[] = array(
            "centralbankname" => $centralbank_read_db->Record['centralbankname'],
                "stateid" => $centralbank_read_db->Record['stateid'],
                "minreserve" => $centralbank_read_db->Record['minreserve'],
                "refinance" => $centralbank_read_db->Record['refinance'],
                "reservebankid" => $centralbank_read_db->Record['reservebankid'],
                "reserveaccountid" => $centralbank_read_db->Record['reserveaccountid'],
                "refinancebankid" => $centralbank_read_db->Record['refinancebankid'],
                "refinanceaccountid" =>  $centralbank_read_db->Record['refinanceaccountid'],
                "centralbankid" => $centralbank_read_db->Record['centralbankid'],
                "centralaccountid" => $centralbank_read_db->Record['centralaccountid']);            
            }
        }
        return  $centralbankparams;
    }
        
    /**
     * Updates the parameters of the central bank
     *
     * @param $secId security id from login
     * @param $lang language for messages
     * @param $centralbankparams parameters of the central bank
     * @return fault or nothing
     */
    function centralbank_update_params($secId,$lang,$centralbankparams)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_centralbankmanager))
        {
            return getFault("2008");
        }
       
        // check user input 
        $result = $this->check_centralbank_params($centralbankparams);
        if(isFault($result))
        {
            return $result;
        }
        
        // get bank parameters from user interface
        $centralbank_update_db = &new DBAccess;
        
        $centralbankname = $centralbankparams["centralbankname"];
        $stateid = $centralbankparams["stateid"]; 
        $minreserve = $centralbankparams["minreserve"];
        $refinance = $centralbankparams["refinance"];
        $reservebankid = $centralbankparams["reservebankid"];
        $reserveaccountid = $centralbankparams["reserveaccountid"];
        $refinancebankid = $centralbankparams["refinancebankid"];
        $refinanceaccountid = $centralbankparams["refinanceaccountid"];
        $centralbankid = $centralbankparams["centralbankid"];
        $centralaccountid = $centralbankparams["centralaccountid"];
                 
        // check bankid and accountid (valid) and kind of account = state
        $check_bankid = "SELECT reservebankid,refinancebankid,centralbankid "
                ."FROM ".$cf_db_prefix."centralbank "
                ."WHERE reservebankid = '".$reservebankid."' "
                ."AND refinancebankid = '".$refinancebankid."' "
                ."AND centralbankid = '".$centralbankid."'";
         
        $centralbank_update_db->query($check_bankid);    
        if($centralbank_update_db->errno != 0)
        {
            return $central_update_db->getError($cf_lang);
        }
        if($centralbank_update_db->num_rows()    == "0")
        {
            $result    = 2062;
            return getFault($result);
        }
        
        $check_accountid = "SELECT a.accountid,b.kindofcustomer "
                ."FROM ".$cf_db_prefix."account AS a,".$cf_db_prefix."customer AS b "
                ."WHERE a.accountid = '".$refinanceaccountid."' "
                ."AND a.accountowner = b.customerid "
                ."AND b.kindofcustomer = '5'";
         
        $centralbank_update_db->query($check_accountid);    
        if($centralbank_update_db->errno != 0)
        {
            return $centralbank_update_db->getError($cf_lang);
        }
        if($centralbank_update_db->num_rows()    == "0")
        {
            $result    = 2063;
            return getFault($result);
        }
        
        $check_accountid = "SELECT a.accountid,b.kindofcustomer "
                ."FROM ".$cf_db_prefix."account AS a,".$cf_db_prefix."customer AS b "
                ."WHERE a.accountid = '".$reserveaccountid."' "
                ."AND a.accountowner = b.customerid "
                ."AND b.kindofcustomer = '5'";
                
        $centralbank_update_db->query($check_accountid);    
        if($centralbank_update_db->errno != 0)
        {
            return $centralbank_update_db->getError($cf_lang);
        }
        if($centralbank_update_db->num_rows()    == "0")
        {
            $result    = 2063;
            return getFault($result);
        }
        
        // accountid of central account 
        $check_accountid = "SELECT a.accountid,b.kindofcustomer "
            ."FROM ".$cf_db_prefix."account AS a,".$cf_db_prefix."customer AS b "
            ."WHERE a.accountid = '".$centralaccountid."' "
            ."AND a.accountowner = b.customerid "
            ."AND b.kindofcustomer = '5'";
         
        $centralbank_update_db->query($check_accountid);    
        if($centralbank_update_db->errno != 0)
        {
            return $centralbank_update_db->getError($cf_lang);
        }
        if($centralbank_update_db->num_rows()    == "0")
        {
            $result    = 2063;
            return getFault($result);
        }                 
        // update bank parameters
        $update_centralbank_params = "UPDATE ".$cf_db_prefix."centralbank " 
                ."SET centralbankname ='".$centralbankname."', "
                ."stateid =  '".$stateid."',"
                ."minreserve = '".$minreserve."',"
                ."refinance = '".$refinance."',"
                ."reservebankid = '".$reservebankid."',"
                ."reserveaccountid = '".$reserveaccountid."',"
                ."refinancebankid = '".$refinancebankid."',"
                ."refinanceaccountid = '".$refinanceaccountid."',"
                ."centralbankid = '".$centralbankid."',"
                ."centralaccountid = '".$centralaccountid."'";

            $centralbank_update_db->query($update_centralbank_params);    
            if($centralbank_update_db->errno != 0)
            {
                return $centralbank_update_db->getError($cf_lang);
            }
            
        }

    //******************************* not public class methods ************************
    
    /**
     * Checks the input of central bank parameters.
     *
     * @param $centralbankparams parameters of the central bank
     * @return fault or nothing
     */
    function check_centralbank_params($centralbankparams)
    {
        $check = array("centralbankname"=>2, "stateid"=>2, "minreserve"=>4,"refinance"=>4,
                         "reservebankid"=>3,"reserveaccountid"=>3, "refinancebankid"=>3,
                          "refinanceaccountid"=> 3);
                         
        $dname = array("centralbankname"=>"Zentralbankname", 
                        "stateid"=>"Staats-Id", 
                        "minreserve"=>"Mindestrcklage",
                        "refinance"=>"Refinanzierungssatz",
                        "reservebankid"=>"Reserve-KontoBLZ",
                        "reserveaccountid"=>"Reserve-Kontonummer",
                        "refinancebankid"=>"Refinanzierungs-KontoBLZ",
                        "refinanceaccountid"=> "Refinanzierungs-Kontonummer");
                          
                                                             
        foreach ($check as $checkname => $checkitem)
        {        
            switch($checkitem)
            {
                case 1:
                    $item = $centralbankparams["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!checkit::isAlpha($item,TRUE))
                    {
                        return getFault("2004",$dname["$checkname"]);
                        break;
                    }
                    break;
                case 2:
                    $item = $centralbankparams["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                            return getFault("2005",$dname["$checkname"]);
                            break;
                    }    
                    break;
                case 3:  
                    $item =$centralbankparams["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$dname["$checkname"]);
                        break;
                    }
                    break;
                case 4:  
                     $item =$centralbankparams["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    } 
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$dname["$checkname"]);
                        break;
                    }
                    if($item < 0 || $item > 100)
                    {
                        return getFault("2061",$dname["$checkname"]);
                        break;
                    }    
                    break;
                default:
                    return getFault("2007");
                    break;
            }    
        }
    }    
    
}
?>