<?php
/**
 * MiNBank business object: customer
 *
 * $Source: /cvsroot/minbank/minba/minba/bo/babo_customer.php,v $
 * $Author: mboller $
 * $Date: 2006/02/13 00:37:01 $
 * $Revision: 1.8 $
 */
 
require_once("../bagn_config.php");
$appl_old = $appl_path;
require_once("$minsoft_path/user/bo/usbo_security.php");
$appl_path = $appl_old;
require_once("$minsoft_path/utility/utdb_access.php");
require_once("$minsoft_path/utility/utgn_config.php");
require_once("$minsoft_path/utility/utrl_role.php");
require_once("$minsoft_path/utility/utgn_fault.php");
require_once("$minsoft_path/utility/utgn_message.php");
require_once("$minsoft_path/utility/utgn_checkit.php");
require_once("$minsoft_path/utility/utxr_client.php");
require_once("$minsoft_path/user/usgn_config.php");

/**
 * Customers of a bank and deputees of customers.
 * Attributes:
 * <table>
 * <tr><td>stateid</td><td></td></tr>
 * <tr><td>bankid</td><td></td></tr>
 * <tr><td>customerid</td><td></td></tr>
 * <tr><td>kindofcustomer</td><td></td></tr>
 * <tr><td>country</td><td></td></tr>
 * <tr><td>town</td><td></td></tr>
 * <tr><td>street</td><td></td></tr>
 * <tr><td>customername</td><td></td></tr>
 * <tr><td>forename</td><td></td></tr>
 * <tr><td>title</td><td></td></tr>
 * <tr><td>customeremail</td><td></td></tr>
 * <tr><td>createdate</td><td></td></tr>
 * <tr><td>deletedate</td><td></td></tr>
 * <tr><td>changedate</td><td></td></tr>
 * <tr><td>locked</td><td></td></tr>
 * <tr><td>deleted</td><td></td></tr>
 * </table>
 */
class Customer
{    
    // **************** class help functions ******************
    /**
     * Gets a word by a name depending on the language.
     *
     * @param $lang language of word
     * @param $name name of word
     */
    function getKeyword($lang, $name) {
        global $minsoft_path; 
        require("$minsoft_path/minba/msg/keywords_".$lang.".txt");
        return $keywords[$name];
    }
    
    /**
     * Checks customer attributes.
     *
     * @param $cust customer values 
     * @return fault or nothing
     */
    function check_customer_params($lang,$cust)
    {
        $check = array("name"=>2, "email"=>4, "custid"=>2,"custart"=>3,
                         "country"=>2,"stateid"=>1, "town"=>2, "street"=>5,
                         "forename"=>5,"title"=>5);
                         
        foreach ($check as $checkname => $checkitem)
        {        
            switch($checkitem)
            {
                case 1: // mandatory alpha
                    $item = $cust["$checkname"];
                    if($item == "")
                    {
                        return getFault("2053",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    else
                    {
                        if(!checkit::isAlpha($item,TRUE))
                        {
                            return getFault("2004",$this->getKeyword($lang, $checkname));
                            break;
                        }
                    }
                    break;
                case 2: // mandatory alphanumeric
                    $item = $cust["$checkname"];
                    if($item == "")
                    {
                        return getFault("2053",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                            return getFault("2005",$this->getKeyword($lang, $checkname));
                            break;
                    }
                    break;
                case 3: // numeric  
                    $item =$cust["$checkname"];
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$this->getKeyword($lang, $checkname));
                        break;
                    }
                    else
                    {
                        if(!($item >0 && $item < 6))
                        {
                            return getFault("2006",$this->getKeyword($lang, $checkname));
                            break;
                        }
                    }     
                    break;
                case 4: // email
                    $item =$cust["$checkname"];
                    if($item == "")
                    {
                        break;
                    }
                    else
                    {
                        if(!checkit::isEmail($item,TRUE))
                        {
                            return getFault("2028",$this->getKeyword($lang, $checkname));
                            break;
                        }
                    }
                    break;
                case 5: // any character, not mandatory
                    if($item == "")
                        break;
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                            return getFault("2005",$this->getKeyword($lang, $checkname));
                            break;
                    }
                            
                        break;
                default:
                    return getFault("2007");
                    break;
            }    
        }
    }
    
    // *************   public class methods ***************
    
    /**
     * Creates a customer.
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $cust array of customer attributes
     * @return fault or nothing
     */
    function create($secId,$lang,$cust)
    {
        global $cf_db_prefix;
        global $uscf_db_prefix;
        
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }

	$deleted=0;
        if(!($us_role & utrl_bankmanager))
        {
	  //return getFault("2008");
	  $deleted=99;
	}


        $result = $this->check_customer_params($lang,$cust);
        if(isFault($result))
        {
            return $result;
        }
        
        $customer_create = &new DBAccess;
        
        $name = $cust["name"];
        $forename = $cust["forename"];
        $title = $cust["title"];
        $custid = $cust["custid"];
        $custart = $cust["custart"];
        $email = $cust["email"];
        $stateid =$cust["stateid"];
        $country = $cust["country"];
        $town = $cust["town"];
        $street = $cust["street"];
        $bankid = $cust["bankid"];
        
        //check if userid in user table
        $check_custid = "SELECT userid from ".$uscf_db_prefix."user ". 
            "WHERE userid = '".$custid."'";
                        
        $customer_create->query($check_custid);
        if($customer_create->errno != 0)
        {
            return $customer_create->getError();
        }
        
        //check if customer is active
        $check_cust = "SELECT customername,deleted "
                        ."FROM ".$cf_db_prefix."customer "
                        ."WHERE customerid = '".$custid."' "
                            ."AND bankid = '".$bankid."'";
                   
        $customer_create->query($check_cust);
        
        if($customer_create->errno != 0)
        {
            return $customer_create->getError();
        }
        if($customer_create->num_rows() == "1")
        {
            $customer_create->next_record();
            $deleted = $customer_create->Record['deleted'];
            if($deleted == '0')
            {
                return getFault("2040");
            }
            // reset deleted sign and insert new values
            $update_cust = "UPDATE ".$cf_db_prefix."customer " 
                    ."SET customerid ='".$custid."',"
                        ."kindofcustomer = '".$custart."',"
                        ."stateid = '".$stateid."',"
                        ."country = '".$country."',"
                        ."town ='".$town."',"
                        ."street ='".$street."',"
                        ."customername ='".$name."',"
                        ."forename='".$forename."',"
                        ."title='".$title."',"
                        ."customeremail ='".$email."',"
                        ."bankid='".$bankid."',"
                        ."deleted='".$deleted."',"
                        ."createdate='".time()."' "
                    ."WHERE customerid = '".$custid."' "
                        ."AND bankid = '".$bankid."'";
            
            $customer_create->query($update_cust);
            if($customer_create->errno != 0)
            {
                return $customer_create->getError();
            }
            
        }
        else
        {   
            // insert into customer table if new customerid
            $insert_cust = "INSERT INTO ".$cf_db_prefix."customer "
                ."(customerid,kindofcustomer, stateid,"
                    ."country,town,street,"
                    ."customername,forename,title,customeremail,"
                    ."bankid,createdate,deleted,locked) "
                ."VALUES"
                    ."('".$custid."','".$custart."',"
                    ."'".$stateid."','".$country."',"
                    ."'".$town."','".$street."','".$name."','".$forename."','".title."',"
                    ."'".$email."','".$bankid."','".time()."','".$deleted."','0')";
    
            $customer_create->query($insert_cust);
            if($customer_create->errno != 0)
            {
                return $customer_create->getError();
            }
        }
    }
    
    /**
     * Gets a list of kinds of customers
     *
     * @param $secId security id
     * @param $lang language of messages
     * @return fault or nothing
     */
    function get_customer_art($secId,$lang)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager))
        {
            return getFault("2008");
        }
        
        $customer_art = &new DBAccess;
        
        //get list
        $get_art = "SELECT customerart,custart_text
                        from ".$cf_db_prefix."kindofcustomer"; 
                    
        $customer_art->query($get_art);
        if($customer_art->errno != 0)
        {
            return $customer_art->getError();
        }
        
        // creat customerart list
        while ($customer_art->next_record())
        {
            $custart[$customer_art->Record["customerart"]] =
            $customer_art->Record["custart_text"];
            
        }
        return $custart;
        
    }
    
    /**
     * Reads a customer.
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $custid customer id
     * @param $bankid bank id
     * @return fault or array of customer attributes
     */
    function read($secId,$lang,$custid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager))
        {
            return getFault("2008");
        }
            
        if($custid == "0")
        {
            return getFault("2014");
        }    
                     
        $customer_read_db = &new DBAccess;
        
        // read customer with customerid and bankid
        $select_cust = "SELECT customerid,kindofcustomer,stateid,"
                    ."country,street,town,"
                    ."customeremail,customername,forename,title,"
                    ."bankid,deleted,locked,createdate,deletedate,changedate "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid = '".$custid."' "
                    ."AND bankid = '".$bankid."'"; 
            
        $customer_read_db->query($select_cust);
        
        if($customer_read_db->errno != 0)
        {
            return $customer_read_db->getError();
        }
        if($customer_read_db->num_rows() != "1")
        {
            $result = 2015;
            return getFault($result);
        }
        else
        {   
        
            // insert customer informaton in array
            $customer_read_db->next_record();
            
            $deleted = $customer_read_db->Record['deleted'];
            if($deleted != 0)
            {
                $result = 2020;
                return getFault($result);
            }
            else
            {
                $cust['custid'] =  $customer_read_db->Record['customerid'];
                $cust['name'] =  $customer_read_db->Record['customername'];
                $cust['forename'] =  $customer_read_db->Record['forename'];
                $cust['title'] =  $customer_read_db->Record['title'];
                $cust['custart'] =  $customer_read_db->Record['kindofcustomer'];
                $cust['stateid'] =  $customer_read_db->Record['stateid'];
                $cust['country'] =  $customer_read_db->Record['country'];
                $cust['town'] =  $customer_read_db->Record['town'];
                $cust['street'] =  $customer_read_db->Record['street'];
                $cust['email'] =  $customer_read_db->Record['customeremail'];
                $cust['bankid'] =  $customer_read_db->Record['bankid'];
                $cust['createdate'] =  $customer_read_db->Record['createdate'];
                $cust['deletedate'] =  $customer_read_db->Record['deletedate'];
                $cust['changedate'] =  $customer_read_db->Record['changedate'];
                $cust['locked'] =  $customer_read_db->Record['locked'];
            }
            
        }
        return  $cust;
        
        
    }
    
    /**
     * Updates a customer.
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $cust array of customer attributes
     * @return fault or nothing
     */
    function update($secId,$lang,$cust)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & (utrl_bankmanager | utrl_customer | utrl_centralbankmanager)))
        {
            return getFault("2008");
        }
        
        $result = $this->check_customer_params($lang,$cust);
        if(isFault($result))
        {
            return $result;
        }
    
        $customer_update_db = &new DBAccess;
        
        $name = $cust["name"];
        $forename = $cust["forename"];
        $title = $cust["title"];
        $custid = $cust["custid"];
        $custart = $cust["custart"];
        $email = $cust["email"];
        $stateid =$cust["stateid"];
        $country = $cust["country"];
        $town = $cust["town"];
        $street = $cust["street"];
        $bankid = $cust["bankid"];
        
        // customer exists?
        $check_cust = "SELECT customername "
                      ."FROM ".$cf_db_prefix."customer WHERE customerid = '".$custid."' "
                      ."AND bankid = '".$bankid."' "
                      ."AND deleted = '0'";
        
        $customer_update_db->query($check_cust);
        
        if($customer_update_db->errno != 0)
        {
            return $customer_update_db->getError();
        }
        if($customer_update_db->num_rows() != "1")
        {
            return getFault(2011);
        }                 
        
        // update customer information in customer table
        $update_cust = "UPDATE ".$cf_db_prefix."customer " 
            ."SET country='".$country."',"
                ."town='".$town."',"
                ."street='".$street."',"
                ."customername='".$name."',"
                ."forename='".$forename."',"
                ."title='".$title."',"
                ."customeremail='".$email."',"
                ."changedate='".time()."' "
            ."WHERE customerid='".$custid."'"
                ."AND bankid='".$bankid."'";
            
        $customer_update_db->query($update_cust);
        
        if($customer_update_db->errno != 0)
        {
            return $customer_update_db->getError();
        }
    }
    
    /**
     * Deletes a customer.
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $custid customer id
     * @param $bankid bank id
     * @return fault or nothing
     */
    function delete($secId,$lang,$custid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager))
        {
            return getFault("2008");
        }
     
        $customer_delete = &new DBAccess;
        
        // customer exists?        
        $select_cust = "SELECT customerid,deleted "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid='".$custid."' "
                    ."AND bankid = '".$bankid."' "
                    ."AND deleted = '0'";
                        
        $customer_delete->query($select_cust);
        if($customer_delete->errno != 0)
        {
            $result = $customer_delete->getError();
            return $result;
        }
        
        // customer does not exist or is deleted
        if($customer_delete->num_rows()== 0)
        {
            $result = 2011;
            return getFault($result);
        
        }
        
        // customer has active accounts ?
        $select_account = "SELECT accountid "
                ."FROM ".$cf_db_prefix."account "
                ."WHERE accountowner='".$custid."' " 
                    ."AND bankid='".$bankid."' "
                    ."AND deleted='0'";   
                                           
        $customer_delete->query($select_account);
        if($customer_delete->errno != 0)
        {
            $result = $customer_delete->getError();
            return $result;
        }
        
        if(!$customer_delete->num_rows()== 0)
        {
            $result = 2026;
            return getFault($result);
        
        }
            
        // set delete sign in customer table 
        $deleted = 1;
        $delete_cust = "UPDATE ".$cf_db_prefix."customer "
                ."SET deleted ='1',"
                    ."deletedate='".time()."' "
                ."WHERE customerid ='".$custid."' "
                ."AND bankid = '".$bankid."'";
            
        $customer_delete->query($delete_cust);
        if($customer_delete->errno != 0)
        {
            return $customer_delete->getError();
        }
    }
    
    /**
     * Reads all deputees belonging to a customer in one bank.
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $customerid id of customer
     * @param $bankid bankid
     * @return fault or list of deputees
     */
    function customer_deputee($secId,$lang,$customerid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        
        if(!($customerid    != "0" ))
        {
            $result    = "2016";
            return getFault($result);
        }            
        $customer_deputee_db = &new    DBAccess;
        $count = 0; 
        $customer_deputee = NULL;
        
        //check if customer
        $check_bank_customer = "SELECT customerid "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid ='".$customerid."' "
                ."AND bankid='".$bankid."' "
                ."AND deleted='0'";
                                
        $customer_deputee_db->query($check_bank_customer);
        if($customer_deputee_db->errno != 0)
        {
            return $customer_deputee_db->getError();
        }
        
        if($customer_deputee_db->num_rows() == "0")
        {
            $customer_deputee = NULL;
        }
        else
        {
            $customer_deputee[$count] = array
            ( "customername"=>"eigene Konten",
            "customerid" => $customerid);
            $count++;
        }
        
        $select_customer_deputee = "SELECT  "
                    .$cf_db_prefix."customer.customername,"
                    .$cf_db_prefix."customer.bankid,"
                    .$cf_db_prefix."deputeecust.deputeeid,"
                    .$cf_db_prefix."deputeecust.customerid "
                    ."FROM ".$cf_db_prefix."deputeecust,".$cf_db_prefix."customer " 
                    ."WHERE ".$cf_db_prefix."deputeecust.deputeeid='".$customerid."' "
                    ."AND ".$cf_db_prefix."deputeecust.bankid='".$bankid."' "
                    ."AND ".$cf_db_prefix."customer.bankid='".$bankid."' "
                    ."AND ".$cf_db_prefix."deputeecust.deleted='0' "
                    ."AND ".$cf_db_prefix."deputeecust.customerid=".$cf_db_prefix."customer.customerid";
                                
        $customer_deputee_db->query($select_customer_deputee);
        if($customer_deputee_db->errno != 0)
        {    
            return $customer_deputee_db->getError();
        }
        
        if(!($customer_deputee_db->num_rows() == "0"))
        {
            // fill list of deputees
            while ($customer_deputee_db->next_record())
            {
                $customer_deputee[$count] = array ("customername"=>
                $customer_deputee_db->Record["customername"],
                "customerid"=>$customer_deputee_db->Record["customerid"],
                "deputeeid"=>$customer_deputee_db->Record["deputeeid"],
                "bankid"=>$customer_deputee_db->Record["bankid"]);        
                $count++;
            }
        }
        return    $customer_deputee;
    }
    
    /**
     * Reads all customers of a deputee
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $customerid id of customer
     * @param $bankid bankid
     * @return fault or list of customers
     */
    function deputee_customer($secId,$lang,$customerid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        if(!($customerid    != "0" ))
        {
            $result    = "2016";
            return getFault($result);
        }            
        $customer_deputee_db = &new    DBAccess;
        $customer_deputee = NULL;
        $count = 0;

        $select_customer_deputee    = "SELECT customerid,bankid,deputeeid ". 
                    "FROM ".$cf_db_prefix."deputeecust ". 
                    "WHERE customerid = '".$customerid."' ".
                    "AND bankid = '".$bankid."' ".
                    "AND deleted = '0'";    
        
        $customer_deputee_db->query($select_customer_deputee);
        if($customer_deputee_db->errno != 0)
        {
            return $customer_deputee_db->getError();

        }
        
        if($customer_deputee_db->num_rows() != 0)
        {

            // fill list of customers
            while ($customer_deputee_db->next_record())
            {
                $customer_deputee[$count] = array (
                        "customerid" => $customerid, 
                        "bankid" => $bankid,
                        "deputeeid" => $customer_deputee_db->Record["deputeeid"]);        
                $count++;
            }
        }
        return    $customer_deputee;
    }
        
    /**
     * Deletes a deputee of a customer.
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $accountowner id of customer
     * @param $bankid bankid
     * @param $deputeeid id of deputee
     * @return fault or nothing
     */
    function customer_deputee_delete($secId,$lang,
            $accountowner,$bankid,$deputeeid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        
        if(!($accountowner    != "0" ))
        {
            $result    = "2016";
            return getFault($result);
        }    
                    
        $customer_deputee_db = &new    DBAccess;
        
        $select_customer_deputee =
                "SELECT customerid,bankid,deputeeid " 
                ."FROM ".$cf_db_prefix."deputeecust " 
                ."WHERE customerid    = '".$accountowner."' "
                    ."AND bankid = '".$bankid."' "
                    ."AND deputeeid = '".$deputeeid."'";
                                
        $customer_deputee_db->query($select_customer_deputee);
        if($customer_deputee_db->errno != 0)
        {
            $result = $customer_deputee_db->errno;
            return getFault($result);
        }
        
        if($customer_deputee_db->num_rows() == "0")
        {
            $result    = 2016;
            return getFault($result);
        }
        
        // customer authorsation?
        $customer_deputee_db->next_record();
        $deputeeid = $customer_deputee_db->Record["deputeeid"];
        if(!($us_role & utrl_bankmanager))
        {
            if(($us_id != $accountowner)&&($us_id != $deputeeid))
            {
                return getFault(2025);
            }
        }
        
        // set delete sign
        $delete_deputee  =
                "UPDATE ".$cf_db_prefix."deputeecust " 
                ."SET deleted='1' "
                ."WHERE customerid='".$accountowner."' "
                    ."AND bankid='".$bankid."' "
                    ."AND deputeeid='".$deputeeid."'";
                        
        $customer_deputee_db->query($delete_deputee);
        if($customer_deputee_db->errno != 0)
        {
            return getFault($customer_deputee_db->errno);
        }
    }
    
    /**
     * Inserts a deputee of a customer.
     *
     * @param $secId security id
     * @param $lang language
     * @param $customerid original customer id
     * @param $bankid bankid of account
     * @param $deputeeid customerid of deputee
     */
    function customer_deputee_new($secId,$lang,
            $customerid,$bankid,$deputeeid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        
        if($accountowner == $deputeeid)
        {
            return getFault("2049");
        }
        if(!($accountowner    != "0" ))
        {
            $result    = "2036";
            return getFault($result);
        }    
        
        $customer_deputee_db = &new    DBAccess;
    
        $select_customer =
                "SELECT customerid "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid = '".$deputeeid."' "
                ."AND deleted = '0'";
                                            
        $customer_deputee_db->query($select_customer);
        if($customer_deputee_db->errno != 0)
        {
            return $customer_deputee_db->getError();

        }
        
        if($customer_deputee_db->num_rows() == "0")
        {                    
            return getFault("2047");
        }
        
        $select_customer_deputee =
                "SELECT customerid,bankid,deputeeid,deleted "
                ."FROM ".$cf_db_prefix."deputeecust "
                ."WHERE customerid = '".$customerid."'"
                    ."AND bankid = '".$bankid."'"
                    ."AND deputeeid = '".$deputeeid."'";            
        
        $customer_deputee_db->query($select_customer_deputee);
        if($customer_deputee_db->errno != 0)
        {
            return $customer_deputee_db->getError();

        }
        if($customer_deputee_db->num_rows() == "0")
        {
            // insert into deputeecust table
            $insert_customer_deputee =
                    "INSERT INTO ".$cf_db_prefix."deputeecust " 
                        ."(customerid,bankid,deputeeid,deleted) "
                    ."VALUES ('".$customerid."','".$bankid."','".$deputeeid."','0')";
                
            $customer_deputee_db->query($insert_customer_deputee);
            if($customer_deputee_db->errno != 0)
            {
                return $customer_deputee_db->getError();
            }
        }
        else
        {
            if($customer_deputee_db->Record['deleted'] == '0')
            {
                return getFault("2044");
            }
            else
            {            
                // reset delete sign
                $update_deputee =
                        "UPDATE ".$cf_db_prefix."deputeecust "
                        ."SET deleted='0' "
                        ."WHERE customerid='".$customerid."' "
                            ."AND bankid = '".$bankid."' "
                            ."AND deputeeid = '".$deputeeid."'";
                        
                $customer_deputee_db->query($update_deputee);
                if($customer_deputee_db->errno != 0)
                {
                    return getFault($customer_deputee_db->errno);
                }
            }
        }
    }
    
    /**
     * Lock or unlock a customer
     *
     * @param $secId
     * @param $lang
     * @param $customerid
     * @param $bankid
     * @param $lock
     */
    function lock($secId,$lang,$customerid,$bankid,$lock) {
        global $cf_db_prefix;
        $cf_lang = $lang;

        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager))
        {
            return getFault("2008");
        }
   
        $customer_update_db = &new DBAccess;
                
        // customer exists?
        $check_cust =
                "SELECT customername "
                    ."FROM ".$cf_db_prefix."customer "
                    ."WHERE customerid = '".$custid."' "
                        ."AND bankid = '".$bankid."' "
                        ."AND deleted = '0'";
        
        $customer_update_db->query($check_cust);
        
        if($customer_update_db->errno != 0)
        {
            return $customer_update_db->getError();
        }
        if($customer_update_db->num_rows() != "1")
        {
            return getFault(2011);
        }                 
        
        // update customer information in customer table
        $update_cust = "UPDATE ".$cf_db_prefix."customer " 
            ."SET country='".$country."',"
                ."town='".$town."',"
                ."street='".$street."',"
                ."customername='".$name."',"
                ."forename='".$forename."',"
                ."title='".$title."',"
                ."customeremail='".$email."',"
                ."changedate='".time()."' "
            ."WHERE customerid='".$custid."'"
            ."AND bankid='".$bankid."'";
            
        $customer_update_db->query($update_cust);
        
        if($customer_update_db->errno != 0)
        {
            return $customer_update_db->getError();
        }        
    }
        
    /**
     * Lists the accounts a customer is authorized to handle.
     * This will not be possible for the customer if the customer
     * is locked.
     *
     * @param $secId security id
     * @param $lang language of messages
     * @param $customerid customer id
     * @param $bankid bank id
     * @return fault or array of accounts aund authorizations
     */
    function account_list($secId,$lang,$customerid,$bankid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
    
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer || $us_role & utrl_technical))
        {
            return getFault("2008");
        }

        $account_list_db = &new    DBAccess;
        
        //check if customer exists?
        $check_customer = "SELECT customerid,customername,forename,title,kindofcustomer,locked "
                ."FROM ".$cf_db_prefix."customer "
                ."WHERE customerid = '".$customerid."' "
                    ."AND bankid ='".$bankid."' "
                    ."AND deleted = '0'";
                              
        $account_list_db->query($check_customer);
        if($account_list_db->errno != 0) 
        {
            return $account_list_db->getError();
        }
        
        if($account_list_db->num_rows() != "1")
        {
            $result = 2015;
            return getFault("2015");
        }                  
        
        $account_list_db->next_record();
        $customername = $account_list_db->Record["customername"];
        $forename = $account_list_db->Record["forename"];
        $title = $account_list_db->Record["title"];
        $kindofcustomer = $account_list_db->Record["kindofcustomer"];
        
        // accounts of accountowner
        $select_accountlist =
                "SELECT accountid,accountowner,accountname,"
                    ."accountbalance,kindofaccount "
                ."FROM ".$cf_db_prefix."account "  
                ."WHERE bankid = '".$bankid."' "
                    ."AND deleted    = '0'"
                    ."AND accountowner = '".$customerid."'";
                
        $account_list_db->query($select_accountlist);
        if($account_list_db->errno != 0) 
        {
            return $account_list_db->getError();
        }
        
        $accountlist =NULL;
        if($account_list_db->num_rows()    > 0)
        {
            
            // fill array with account list
            $count = 0;
            while ($account_list_db->next_record())
            {
                
                $accountlist[$count] = array(
                        "accountid"=>$account_list_db->Record['accountid'],
                        "customerid"=>$account_list_db->Record['accountowner'],
                        "accountname"=>$account_list_db->Record['accountname'],
                        "customername"=>$customername,
                        "forename"=>$account_list_db->Record['forename'],
                        "title"=>$account_list_db->Record['title'],
                        "amount"=>$account_list_db->Record['accountbalance'],
                        "kindofcustomer"=>$kindofcustomer);
                        $count++;
                                                        
            }
        }
        
        // get authorized accounts
        $select_accountlist =
                "SELECT a.accountid, a.accountowner, "
                    ."a.accountbalance,a.kindofaccount,"
                    ."c.customername,c.forename,c.title,c.kindofcustomer "
                    ."FROM ".$cf_db_prefix."account AS a,"
                        .$cf_db_prefix."deputeeaccount AS b,"
                        .$cf_db_prefix."customer AS c "
                            ."WHERE a.accountid = b.accountid "
                            ."AND b.deputeeid = '".$customerid."' "
                            ."AND c.customerid = a.accountowner "
                            ."AND b.bankid = a.bankid "
                            ."AND c.bankid = a.bankid "
                            ."AND b.bankid = '".$bankid."' "
                            ."AND b.deleted = '0'"; 
                                    
        $account_list_db->query($select_accountlist);                            
        if($account_list_db->errno != 0) 
        {
            return $account_list_db->getError();
        }
        
        $authlist = NULL;
        if($account_list_db->num_rows()    > 0)
        {
            
            // fill array with account list
            $count = 0;
            while ($account_list_db->next_record())
            {
                $customername = $account_list_db->Record["customername"];
                $kindofcustomer = $account_list_db->Record["kindofcustomer"];
                $authlist[] = array(
                        "accountid"=>$account_list_db->Record['accountid'],
                        "customerid"=>$account_list_db->Record['accountowner'],
                        "customername"=>$account_list_db->Record['customername'],
                        "forename"=>$account_list_db->Record['forename'],
                        "title"=>$account_list_db->Record['title'],
                        "amount"=>$account_list_db->Record['accountbalance'],
                        "kindofcustomer"=>$account_list_db->Record['kindofcustomer']);
                        $count++;
                                
            }
        }
                    
        $daccountlist = array("account"=>$accountlist,"auth"=>$authlist);
        return    $daccountlist;
    }    
}      
?>