<?php
/**
 * MiNBank business object: economic
 *
 * $Source: /cvsroot/minbank/minba/minba/bo/babo_economic.php,v $
 * $Author: eltres $
 * $Date: 2006/03/10 21:39:20 $
 * $Revision: 1.4 $
 */

require_once("../bagn_config.php");
$appl_old = $appl_path;
require_once("$minsoft_path/user/bo/usbo_security.php");
$appl_path = $appl_old;
require_once("$appl_path/bo/babo_globmeth.php");
require_once("$minsoft_path/utility/utdb_access.php");
require_once("$minsoft_path/utility/utgn_config.php");
require_once("$minsoft_path/utility/utrl_role.php");
require_once("$minsoft_path/utility/utgn_checkit.php");
require_once("$minsoft_path/utility/utxr_client.php");

/**
 * Economy of a country. Responsible for these values should be the ministry
 * of economics of a country.
 * <p>
 * The country economy has the following attributes:
 *   <table>
 *   <tr><td>countryname</td><td>name of the country</td></tr>
 *   <tr><td>stateid</td><td>short id of the country, used for international transactions</td></tr>
 *   <tr><td>currname</td><td>long name of the currency</td></tr>
 *   <tr><td>currsign</td><td>short currency sign</td></tr>
 *   <tr><td>conduty</td><td>duty in percent to be payed of no consume is done</td></tr>
 *   <tr><td>turnovertax</td><td>tax</td></tr>
 *   <tr><td>interesttax</td><td>tax for credits</td></tr>
 *   <tr><td>incometax</td><td>tax in percentage on incomes</td></tr>
 *   <tr><td>bankid</td><td>bankid for taxes</td></tr>
 *   <tr><td>taxaccountid</td><td>account id for taxes</td></tr>
 *   <tr><td>admin_date</td><td>date of last changes</td></tr>
 *   </table>
 */
class Economic
{    
        
    /**
     * Reads the attributes of country economics.
     *
     * @access public
     * @param $secID security id from login
     * @param $lang language for messages
     * @return array of attributes or fault
     */
    function economic_read_params($secId,$lang)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_mineconomics))
        {
            return getFault("2008");
        }
       
        $economic_read_db = &new DBAccess;
            
        // read parameters
        $select_economic = "SELECT  * 
            from {$cf_db_prefix}countryeconomy";
            
           
        $economic_read_db->query($select_economic);
        if($economic_read_db->errno != 0)
        {
            return $economic_read_db->getError();
        }
        if($economic_read_db->num_rows() == 0)
        {
            $economic = NULL;
            return getFault("2013");
            
        }
        else
        {   
        
            // fill array with centralbank params
            while ($economic_read_db->next_record())
            {
                $economic[] = array(
                "countryname" => $economic_read_db->Record['countryname'],
                "stateid" => $economic_read_db->Record['stateid'],
                "currname" => $economic_read_db->Record['currname'],
                "currsign" => $economic_read_db->Record['currsign'],
                "conduty" => $economic_read_db->Record['conduty'],
                "turnovertax" => $economic_read_db->Record['turnovertax'],
                "interesttax" => $economic_read_db->Record['interesttax'],
                "incometax" => $economic_read_db->Record['incometax'],
                "bankid" =>  $economic_read_db->Record['bankid'],
                "taxaccountid" => $economic_read_db->Record['taxaccountid'],
                "admin_date" => $economic_read_db->Record['admin_date']);            
            }
        }
        return  $economic;
    }
        
    /**
     * Updates the attributes of the economy of a country.
     *
     * @access public
     * @param $secId security id
     * @param $lang language for messages
     * @param $economic
     * @return nothing or fault
     */
    function economic_update_params($secId,$lang,$economic)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_mineconomics))
        {
            return getFault("2008");
        }
       
       
        // check user input 
        $result = $this->check_economic_attributs($economic);
        if(isFault($result))
        {
            return $result;
        }
        
        // get state economic parameters from user interface
        $economic_update_db = &new DBAccess;
        
        $countryname = $economic["countryname"];
        $stateid = $economic["stateid"]; 
        $currname = $economic["currname"];
        $currsign = $economic["currsign"];
        $conduty  = $economic["conduty"];
        $turnovertax = $economic["turnovertax"];
        $interesttax = $economic["interesttax"];
        $incometax = $economic["incometax"];
        $bankid = $economic["bankid"];
        $taxaccountid = $economic["taxaccountid"];
        $admin_date = $economic["admin_date"];
        $admin_time = $economic["admin_time"];
        
        // check bankid and accountid (valid) and kind of account = state
	/*  $check_bankid = "SELECT bankid from {$cf_db_prefix}countryeconomy " 
                        ."WHERE bankid='".$bankid."'";
         
        $economic_update_db->query($check_bankid);    
        if($economic_update_db->errno != 0)
        {
            return $economic_update_db->getError($cf_lang);
        }
        if($economic_update_db->num_rows()    == "0")
        {
            $result    = 2062;
            return getFault($result);
        }
	
        
        $check_accountid = "SELECT a.accountid,b.kindofcustomer from {$cf_db_prefix}account AS a, "
            ."{$cf_db_prefix}customer AS b "
            ."WHERE accountid = '".$taxaccountid."' "
            ."AND a.accountowner=b.customerid "
            ."AND b.kindofcustomer = '5'";
         
        $economic_update_db->query($check_accountid);    
        if($economic_update_db->errno != 0)
        {
            return $economic_update_db->getError($cf_lang);
        }
        if($economic_update_db->num_rows()    == "0")
        {
            $result    = 2063;
            return getFault($result);
        }
        */
        $day = substr($admin_date,0,2);
        $month = substr($admin_date,2,2);
        $year = substr($admin_date,4,4);
        $hour =substr($admin_time,0,2);
        $minute = substr($admin_time,2,2);
        $sec = substr($admin_time,4,2);
                    
        $date = mktime($hour,$minute,$sec,$month,$day,$year);
                          
        // update bank parameters
        $update_economic_params = "UPDATE {$cf_db_prefix}countryeconomy "
                ."SET countryname ='".$countryname."',"
                ."stateid='".$stateid."',"
                ."currname='".$currname."',"
                ."currsign='".$currsign."',"
                ."conduty='".$conduty."',"
                ."turnovertax='".$turnovertax."',"
                ."interesttax='".$interesttax."',"
                ."incometax='".$incometax."',"
                ."bankid='".$bankid."',"
                ."taxaccountid='".$taxaccountid."',"
                ."admin_date='".$date."'";
      
            $economic_update_db->query($update_economic_params);    
            if($economic_update_db->errno != 0)
            {
                return $economic_update_db->getError($cf_lang);
            }
            
        }

    //******************************* not public class methods ************************
    
    /**
     * Checks values of economic attributs array.
     *
     * @access private
     * @param $economic array of economic attributes
     * @return nothing or fault
     */
    function check_economic_attributs($economic)
    {
        $check = array("countryname"=>1, "stateid"=>2, "currname"=>1,"currsign"=>2,  "admin_date" => 5,"admin_time" => 6);
                         
        $dname = array("countryname"=>"Staatsname", 
                        "stateid"=>"Staats-Id", 
                        "currname"=>"Whrungsname",
                        "currsign"=>"Whrungs-Id.",
                        "admin_date" => "Verwaltungs-Datum",
                        "admin_time" => "Verwaltungs-Uhrzeit");
                          
                                                             
        foreach ($check as $checkname => $checkitem)
        {        
            switch($checkitem)
            {
                case 1:
                    $item = $economic["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!checkit::isAlpha($item,TRUE))
                    {
                        return getFault("2004",$dname["$checkname"]);
                        break;
                    }
                    break;
                case 2:
                    $item = $economic["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!checkit::isAlphaNum($item,TRUE))
                    {
                            return getFault("2005",$dname["$checkname"]);
                            break;
                    }    
                    break;
                case 3:  
                    $item =$economic["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$dname["$checkname"]);
                        break;
                    }
                    break;
                case 4:  
                    $item =$economic["$checkname"];
                    if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    } 
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$dname["$checkname"]);
                        break;
                    }
                    if($item < 0 || $item > 100)
                    {
                        return getFault("2061",$dname["$checkname"]);
                        break;
                    }    
                    break;
                case 5:  
                    $item =$economic["$checkname"];
                     if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$dname["$checkname"]);
                        break;
                    }
                    // transform and check date
                    $day = substr($item,0,2);
                    $month = substr($item,2,2);
                    $year = substr($item,4,4);
                    if($day < 1 || $day > 31)
                    {
                        return getFault("2064",$dname["$checkname"]);
                        break;
                    }
                    if($month < 1 || $month > 12)
                    {
                        return getFault("2064",$dname["$checkname"]);
                        break;
                    }
                    if($year < 2003 || $year > 9999)
                    {
                        return getFault("2064",$dname["$checkname"]);
                        break;
                    }
                    break;
                case 6:  
                    $item =$economic["$checkname"];
                     if($item == "")
                    {
                        return getFault("2052",$dname["$checkname"]);
                        break;
                    }
                    if(!($item != "" && is_numeric($item)))
                    {
                        return getFault("2006",$dname["$checkname"]);
                        break;
                    }
                    // transform and check date
                    $hour = substr($item,0,2);
                    $minute = substr($item,2,2);
                    $sec = substr($item,4,4);
                    if($hour < 0 || $hour > 23)
                    {
                        return getFault("2064",$dname["$checkname"]);
                        break;
                    }
                    if($minute < 0 || $minute > 59)
                    {
                        return getFault("2064",$dname["$checkname"]);
                        break;
                    }
                    if($sec < 0 || $sec > 59)
                    {
                        return getFault("2064",$dname["$checkname"]);
                        break;
                    }
                    break;
                default:
                    return getFault("2007");
                    break;
            }    
        }
    }    
}
?>