<?php
/**
 * MiNBank business object: standing order
 *
 * $Source: /cvsroot/minbank/minba/minba/bo/babo_standingorder.php,v $
 * $Author: eltres $
 * $Date: 2007/01/21 01:21:01 $
 * $Revision: 1.15 $
 */

require_once("../bagn_config.php");
require_once("$appl_path/bo/babo_globmeth.php");
require_once("$minsoft_path/utility/utdb_access.php");
require_once("$minsoft_path/utility/utgn_config.php");
require_once("$minsoft_path/utility/utgn_trace.php");
require_once("$minsoft_path/utility/utgn_fault.php");
require_once("$minsoft_path/utility/utgn_message.php");
require_once("$minsoft_path/utility/utrl_role.php");
require_once("$minsoft_path/utility/utgn_checkit.php");
require_once("$minsoft_path/utility/utxr_client.php");
require_once("$minsoft_path/minba/$cf_local_interface/bali_account.php");


global $girokonto;

/**
 * Perfoms the adminstration of standing orders.
 * <p>
 * Attributes of standing orders are:
 * <table>
 * <tr><td>remitterbankid</td><td>Bank id of remitter.</td></tr>
 * <tr><td>remitteraccountid</td><td>Account id of remitter.</td></tr>
 * <tr><td>payeebankid</td><td>Bank id of payee.</td></tr>
 * <tr><td>payeeaccountid</td><td>Account id of payee.</td></tr>
 * <tr><td>customerid</td><td>Customer id of remitter.</td></tr>
 * <tr><td>purpose</td><td>Purpose of money transfer.</td></tr>
 * <tr><td>currencyid</td><td>Name if currency.</td></tr>
 * <tr><td>amount</td><td>Amount of money to transfer. Decimal separator has to be a point.</td></tr>
 * <tr><td>payeename</td><td>Name of payee.</td></tr>
 * <tr><td>period</td><td>Time period. 0=daily,1=weekly,2=monthly.</td></tr>
 * <tr><td>time</td><td>Time of transfer. Weekday for weekly (0=monday). day of month for monthly.</td>
 * <tr><td>nextexecution</td><td>Time of next execution as seconds since 1970.</td></tr>
 * <tr><td>lastexecuted</td><td>Time of last execution as seconds since 1970.</td></tr>
 *</table>
 */
class StandingOrder
{    
    //*****************   private help functions ****************************
    
    /**
     * Checks the user input of standing orders.
     *
     * @access private
     * @param standingOrder standing order attribute array
     * @return fault or nothing=OK
     */
    function check_standingorder_params($standingOrder)
    {
        $check = array(
            "remitterbankid"=>1,
            "remitteraccountid"=>1,
            "payeebankid"=>1,
            "payeeaccountid"=>1,
            "amount"=>2,
            "purpose"=>1,
            "customerid"=>1,
            "period"=>3,
            "time"=>4,
            "nextexecution"=>1);
                         
        $namelist = array(
                "remitterbankid"=>"von Bank",
                "remitteraccountid"=>"von Konto",
                "payeebankid"=>"Bankleitzahl",
                "payeeaccountid"=>"Konto-nr.",
                "amount"=>"Betrag",
                "purpose"=>"Verwendungszweck",
                "customerid"=>"Beg&uuml;nstigter",
                "period"=>"H&auml;ufigkeit",
                "time"=>"Zeitpunkt",
                "lastexecuted"=>"n&auml;chste Ausf&uuml;hrung",
                );
         
        // check for transfer to same account       
        if($standingOrder['remitterbankid'] == $standingOrder['payeebankid'] &&
                $standingOrder['remitteraccountid'] == $standingOrder['payeeaccountid']) {
            return getFault("2078");
        }
                                             
        foreach ($check as $checkname => $checkitem)
        {
            switch($checkitem)
            {
                case 1: // not empty, alphanumeric
                    $item = $standingOrder[$checkname];
                    if($item == "") {
                        return getFault("2053",$namelist["$checkname"]);
                    }
                    if(!checkit::isAlphaNum($item,TRUE)) {
                        return getFault("2005",$namelist["$checkname"]);
                    }
                    break;
                case 2:  // not empty, numeric
                    $item = $standingOrder[$checkname];
                    if(!($item != "" && is_numeric($item))) {
                        return getFault("2006",$namelist[$checkname]);
                    }
                    break;     
                case 3: // period
                    $item = $standingOrder[$checkname];
                    if($item == "" || ! is_numeric($item)) {
                        return getFault("2006",$namelist[$checkname]);
                    }
                    if($item < 0 || $item > 2) {
                        return getFault("2070",$namelist[$checkname]);
                    }
                    break;     
                case 4: // time
                    if($standingOrder["period"] != 0) {
                        $item = $standingOrder[$checkname];
                        if($item == "" || ! is_numeric($item))
                            return getFault("2006",$namelist["$checkname"]);
                    }
                    switch($standingOrder["period"]) {
                        case 0: // daily
                            // no further information
                            break;
                        case 1: // weekly
                            if($item < 0 || $item > 6) {
                                return getFault("2071",$namelist[$checkname]);
                            }
                            break;
                        case 2: // monthly
                            if($item < 0 || $item > 30) {
                                return getFault("2071",$namelist[$checkname]);
                            }
                            break;
                    }
                    break;     
                default:
                    return getFault("2007");
                    break;
            }    
        }
    }

    /**
     * Checks if it is a bankmanager, owner, account deputee or customer deputee.
     *
     * @access private
     * @param accountid id of account
     * @param bankid id of bank
     * @param us_id user id
     * @param us_role user role
     * @param prefix database table name prefix
     * @param lang language
     * @param account account transfer parameters
     * @return fault or empty=OK
     */
    function check_authorities($accountid,$bankid,$us_id,$us_role,$prefix)
    {
        $auth_check_db = &new DBAccess;
        $result = NULL;

        // get the account owner
        $select_owner = "SELECT accountowner "
                ."FROM ".$prefix."account "
                ."WHERE accountid = '".$accountid."' "
                ."AND bankid ='".$bankid."' "
                ."AND deleted = '0'";

         $auth_check_db->query($select_owner);
         if($auth_check_db->errno != 0)
         {
            return $auth_check_db->getError();
         }

         if($auth_check_db->num_rows() != "1")
         {
             //account does not exist
             return getFault("2016");
        }

        $auth_check_db->next_record();
        $accountowner = $auth_check_db->Record["accountowner"];

        // check if account id is a deputee or customer deputee
        if(!($us_role & utrl_bankmanager))
        {
            if($us_id != $accountowner)
            {
                $select_deputee = "SELECT deputeeid "
                        ."FROM ".$prefix."deputeeaccount "
                        ."WHERE accountid = '".$accountid."' "
                        ."AND deputeeid = '".$us_id."' "
                        ."AND deleted = '0'";

                $auth_check_db->query($select_deputee);
                if($auth_check_db->errno != 0)
                {
                    return $auth_check_db->getError();
                }

                if($auth_check_db->num_rows() == "0")
                {
                    // check if customer is a deputee ?
                    $select_authcust = "SELECT deputeeid "
                            ."FROM ".$prefix."deputeecust "
                            ."WHERE deputeeid ='".$us_id."' "
                            ."AND bankid = '".$bankid."' "
                            ."AND deleted = '0' "
                            ."AND customerid = '".$accountowner."'";

                    $auth_check_db->query($select_authcust);
                    if($auth_check_db->errno != 0)
                    {
                        $result    = $auth_check_db->getError();
                        return $result;
                    }
                    if($auth_check_db->num_rows()== 0)
                    {
                        return getFault("2016");
                    }
                 }
            }
        }
        return $result;
    }
    
    //*************    public class methods *************************
    
    /**
     * Creates a standing order.
     *
     * @access public
     * @param secId security id from login
     * @param lang language
     * @param standingorder standing order attributes
     * @return fault or new standing order id
     */
    function create($secId, $lang, $standingorder)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        
        // check parameters
        $result    = $this->check_standingorder_params($standingorder);
        if(isFault($result)) {
            return $result;
        }
        
        // customerid exists? and is active?
        $accountowner = $standingorder["customerid"];
        $bankid = $standingorder["remitterbankid"];
        $result= get_custid($accountowner,$bankid);
        
        if($result != 1)
        {
            return getFault(2014);
        }

        // a customer administers only his own standing orders
        // a bankmanager administers all standing orders belonging to the bank
//         if(!($us_role & utrl_bankmanager) && $accountowner != $us_id)
//         {
//             return getFault(2024);
//         }

        // adjust next execution date
        $nextexecution = $this->adjustNextExecution(
                $standingorder['period'],
                $standingorder['time'],
                $standingorder['nextexecution']);
        
        // insert into standingorder table
        $sql = "INSERT INTO ".$cf_db_prefix."standingorder "
                ."(remitterbankid,remitteraccountid,payeebankid,payeeaccountid,customerid,"
                ."purpose,currencyid,amount,payeename,period,time,nextexecution,lastexecuted) "
                ."VALUES("
                ."'".$standingorder['remitterbankid']."',"
                ."'".$standingorder['remitteraccountid']."',"
                ."'".$standingorder['payeebankid']."',"
                ."'".$standingorder['payeeaccountid']."',"
                ."'".$standingorder['customerid']."',"
                ."'".$standingorder['purpose']."',"
                ."'".$standingorder['currencyid']."',"
                ."'".$standingorder['amount']."',"
                ."'".$standingorder['payeename']."',"
                ."'".$standingorder['period']."',"
                ."'".$standingorder['time']."',"
                ."'".$nextexecution."',"
                ."'0'"
                .")";
    
        $standingorder_create = &new DBAccess;
        $standingorder_create->query($sql);
        if($standingorder_create->errno != 0)
        {
            return $standingorder_create->getError();
        }
        
        // account id will be delivered by autoincrement
        // the mysql function "max" does not work!!!
        $standingorderid = mysql_insert_id();
        return $standingorderid;
        
    }
    
    /**
     * Reads standingorder information
     *
     * @access public
     * @param secId security id from login
     * @param lang language
     * @param standingorderid standing order id
     * @return fault or standing order attribute error
     */
    function read($secId, $lang, $soid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        // check security
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault(2008);
        }
        // check input
        if($soid == 0 || ! is_numeric($soid))
        {
            return getFault(2006);
        }    
                
        // 
        $standingorder_read_db = &new DBAccess;
        
        // select account parameter values            
        $select_standingorder = "SELECT soid,"
                ."remitterbankid," 
                ."remitteraccountid," 
                ."payeebankid,"
                ."payeeaccountid," 
                ."customerid,"
                ."purpose," 
                ."currencyid,"
                ."amount," 
                ."payeename," 
                ."period," 
                ."time," 
                ."nextexecution," 
                ."lastexecuted " 
                ."FROM ".$cf_db_prefix."standingorder "    
                ."WHERE soid='".$soid."' ";
        $standingorder_read_db->query($select_standingorder);
        if($standingorder_read_db->errno != 0)
        {
            return $standingorder_read_db->getError();
        }
        if($standingorder_read_db->num_rows() != "1")
        {
            return getFault(2074);
        }
        else
        {
            $standingorder_read_db->next_record();
            
            // customer or deputee valid?
            $result = $this->check_authorities(
                $standingorder_read_db->Record['remitteraccountid'],
                $standingorder_read_db->Record['remitterbankid'],
                $us_id,$us_role,$cf_db_prefix);
    
            if(isFault($result))
            {
                return $result;
            }
          
            // fill array with standing order information
            $deleted = $standingorder_read_db->Record['deleted'];
            $standingorder['soid']              = $standingorder_read_db->Record['soid'];
            $standingorder['remitterbankid']    = $standingorder_read_db->Record['remitterbankid'];
            $standingorder['remitteraccountid'] = $standingorder_read_db->Record['remitteraccountid'];
            $standingorder['payeebankid']       = $standingorder_read_db->Record['payeebankid'];
            $standingorder['payeeaccountid']    = $standingorder_read_db->Record['payeeaccountid'];
            $standingorder['customerid']        = $standingorder_read_db->Record['customerid'];
            $standingorder['purpose']           = $standingorder_read_db->Record['purpose'];
            $standingorder['currencyid']        = $standingorder_read_db->Record['currencyid'];
            $standingorder['amount']            = $standingorder_read_db->Record['amount'];
            $standingorder['payeename']         = $standingorder_read_db->Record['payeename'];
            $standingorder['period']            = $standingorder_read_db->Record['period'];
            $standingorder['time']              = $standingorder_read_db->Record['time'];
            $standingorder['nextexecution']     = $standingorder_read_db->Record['nextexecution'];
            $standingorder['lastexecuted']      = $standingorder_read_db->Record['lastexecuted'];
        }
        return $standingorder;
    }
    
    /**
     * Deletes a standing order.
     *
     * @access public
     * @param secId security id from login
     * @param lang language
     * @param standingorderid standing order id
     * @return fault or nothing
     */
    function delete($secId,$lang,$soid)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
         
        $so_delete = &new DBAccess;
        
        // standing order exists?
        $select_so    = "SELECT remitteraccountid,remitterbankid "
                ."FROM ".$cf_db_prefix."standingorder "
                ."WHERE soid='".$soid."'";
                        
        $so_delete->query($select_so);
        if($so_delete->errno != 0)
        {
            return $so_delete->getError();
        }
        if($so_delete->num_rows() == 0)
        {
            return getFault(2074);
        }

        $so_delete->next_record();
        
        // customer or deputee valid?
        $accountid = $so_delete->Record['remitteraccountid'];
        $bankid = $so_delete->Record['remitterbankid'];
        $result = $this->check_authorities($accountid,$bankid,
                $us_id,$us_role,$cf_db_prefix);

        if(isFault($result))
        {
            return $result;
        }
      
        // set delete sign
        $sql = "DELETE FROM ".$cf_db_prefix."standingorder "
                ."WHERE soid='".$soid."'";
                        
        $so_delete->query($sql);
        if($so_delete->errno != 0)
        {
            return $so_delete->getError();
        }
    }
    
    /**
     * Updates a standing order.
     *
     * @access public
     * @param secId security id from login
     * @param lang language
     * @param standingorder standing order attributes
     * @return fault or nothing
     */
    function update($secId,$lang,$standingorder)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        if(!($us_role & utrl_bankmanager || $us_role & utrl_customer))
        {
            return getFault("2008");
        }
        
        // check parameters
        $result = $this->check_standingorder_params($standingorder);
        if(isFault($result)) {
            return $result;
        }
        $standingorder_update = &new DBAccess;
                
        // customerid exists? and is active?
        $accountowner = $standingorder["customerid"];
        $bankid = $standingorder["remitterbankid"];
        $result= get_custid($accountowner,$bankid);
        
        if($result != 1)
        {
            return getFault(2014);
        }
        
        // a customer administers only his own standing orders
        // a bankmanager administers all standing orders belonging to the bank
        if(!($us_role & utrl_bankmanager) && $accountowner != $us_id)
        {
            return getFault(2024);
        }
        
        // adjust next execution date
        $nextexecution = $this->adjustNextExecution(
                $standingorder['period'],
                $standingorder['time'],
                $standingorder['nextexecution']);
        
        // update standingorder without last execution
        $sql = "UPDATE ".$cf_db_prefix."standingorder "
                ."SET remitterbankid='".$standingorder['remitterbankid']."',"
                ."remitteraccountid='" .$standingorder['remitteraccountid']."',"
                ."payeebankid='"       .$standingorder['payeebankid']."',"
                ."payeeaccountid='"    .$standingorder['payeeaccountid']."',"
                ."customerid='"        .$standingorder['customerid']."',"
                ."purpose='"           .$standingorder['purpose']."',"
                ."currencyid='"        .$standingorder['currencyid']."',"
                ."amount='"            .$standingorder['amount']."',"
                ."payeename='"         .$standingorder['payeename']."',"
                ."period='"            .$standingorder['period']."',"
                ."time='"              .$standingorder['time']."',"
                ."nextexecution='"     .$nextexecution."',"
                ."lastexecuted='"      .$standingorder['lastexecuted']."'"
                ."WHERE soid='"        .$standingorder['soid']."'";
    
        $standingorder_update = &new DBAccess;
        $standingorder_update->query($sql);
        if($standingorder_update->errno != 0)
        {
            return $standingorder_update->getError();
        }        
    }

    /**
     * Adjusts value of next execution given by user.
     *
     * @access private
     * @param $period period of execution: daily=0, weekly=1, monthly=2
     * @param $time time when to execute
     * @param $nextexecution time of next execution as given by user
     * @return real next execution time
     */
    function adjustNextExecution($period, $time, $nextExecution) {
        switch($period) {
            case 0: // daily
                // nothing to do
                break;
            case 1: // weekly
                $nextArray = getdate($nextExecution);
                $nodays = (8 + $time - $nextArray['wday']) % 7;
                if($nodays != 0)
                    $nextExecution = mktime(0,0,0,$nextArray['mon'],$nextArray['mday']+$nodays,$nextArray['year']);
                break;
            case 2: // monthly
                $nextArray = getdate($nextExecution);
                $lastmonthday = mktime(0,0,0,$nextArray['mon']+1,0,$nextArray['year']);
                $time++;
                if($time < $nextArray['mday']) {
                    $lastmonthday = mktime(0,0,0,$nextArray['mon']+2,0,$nextArray['year']);
                    $lastArray = getdate($lastmonthday);
                    if($time > $lastArray['mday'])
                        $time = $lastArray['mday'];
                    $nextExecution = mktime(0,0,0,$nextArray['mon']+1,$time,$nextArray['year']);
                }
                else {
                    $lastmonthday = mktime(0,0,0,$nextArray['mon']+1,0,$nextArray['year']);
                    $lastArray = getdate($lastmonthday);
                    if($time > $lastArray['mday'])
                        $time = $lastArray['mday'];
                    $nextExecution = mktime(0,0,0,$nextArray['mon'],$time,$nextArray['year']);
                }
                break;
        }
        return $nextExecution;
    }
    
    /**
     * Evaluates next execution time.
     *
     * @access private
     * @param $period period of execution: daily=0, weekly=1, monthly=2
     * @param $time time when to execute
     * @param $lastexecuted date for which function has been executed the last time
     * @return next execution time
     */
    function evalnextexecution($period, $time, $lastexecuted) {
        $lastArray = getdate($lastexecuted);
        switch($period)
        {
            case 0: // daily 
                $nextexecution = mktime(0,0,0,$lastArray['mon'],$lastArray['mday']+1,$lastArray['year']);
                break;
            case 1: // weekly
                // compute number of days till next target week day
                $nodays = (8 + $time - $lastArray['wday']) % 7;
                if($nodays == 0)
                    $nodays = 7;
                $nextexecution = mktime(0,0,0,$lastArray['mon'],$lastArray['mday']+$nodays,$lastArray['year']); 
                break;
            case 2: // monthly
                // if suggested day is larger than month, then set day to last of month
                $time++;
                $lastmonthday = mktime(0,0,0,$lastArray['mon']+2,0,$lastArray['year']);
                $lastdayArray = getdate($lastmonthday);
                if($time > $lastdayArray['mday'])
                    $time = $lastdayArray['mday'];

                $nextexecution = mktime(0,0,0,$lastArray['mon']+1,$time,$lastArray['year']); 
                    
                break;
        }
        return $nextexecution;
    }
    
    /**
     * Executes all pending standing orders. Transfers are generated for them.
     * The lastexecution attribute is updated. The current implementation only
     * generates one execution for each standing order, this should be extended
     * to generate multiple standing orders if the execution has been omitted
     * several times.
     * <p>
     * On errors in single executions the error is logged without terminating
     * the overall execution.
     *
     * @access public
     * @param secId security id from login
     * @param lang language
     * @return fault or number of standing orders executed
     */
    function execute($secId,$lang)
    {
        global $cf_db_prefix;
        $cf_lang = $lang;
        
        // check security id
        $secResult = checkSecurity($secId);
        if(isFault($secResult))
        {
            return $secResult;
        }
        else
        {
            $us_id = $secResult["userid"];
            $us_role = $secResult["role"];    
        }
        
        // execution will run under technical account or central bank account
        if(!($us_role & (utrl_technical | utrl_centralbankmanager)))
        {
            return getFault(2008);
        }
        
        $so_execute = &new DBAccess;
         
        // get all standing orders
        $sql = "SELECT * "
                ."FROM ".$cf_db_prefix."standingorder";
    
        $so_execute->query($sql);
        if($so_execute->errno != 0)
        {
            return $so_execute->getError();
        }
        
        if($so_execute->errno != 0)
        {
            return $so_execute->getError();
        }

        // initialize number of transfers executed
        $noso = 0;

        // evaluate next execution time
        $lastexecuted = $so_execute->Record['lastexecuted'];

        $accountLi = &new AccountLi;
         
        // for all standing orders
        while($so_execute->next_record()) {
            
            $period = $so_execute->Record['period'];
            $time = $so_execute->Record['time'];
            $lastexecuted = $so_execute->Record['lastexecuted'];
            $nextexecution = $so_execute->Record['nextexecution'];
            
            $today = time();
            $errorflag = false;
            
            // while execution are necessary            
            while($nextexecution <= $today && ! $errorflag) {
                $execdate = date('d.m.Y', $nextexecution);

                // generate transfer for standing order
                $transfer = array(
                    "remitteraccountid"    => $so_execute->Record['remitteraccountid'],
                    "remitterbankid"       => $so_execute->Record['remitterbankid'],
                    "remitteraccountowner" => $so_execute->Record['customerid'],
                    "payeeaccountid"       => $so_execute->Record['payeeaccountid'],
                    "payeebankid"          => $so_execute->Record['payeebankid'],
                    "payeeaccountowner"    => $so_execute->Record['payeename'],
                    "accountcurrency"      => $so_execute->Record['currencyid'],
                    "amount"               => $so_execute->Record['amount'],
                    "purpose"              => $so_execute->Record['purpose'].','.$execdate
                );

                // remember last date for which function has been executed                
                $lastexecuted = $nextexecution;
                
                // evaluate new next execution
                $nextexecution = $this->evalnextexecution($period, $time, $nextexecution);
                $accountLi->setStandingOrder();
                $result = $accountLi->transfer_process($secId,$lang,$transfer);
                if(isFault($result))
                {
                   utgn_trace("so_execute", $result[0]['faultString']);
                   $errorflag = true;
                }
                else {
                    $so_update = &new DBAccess;
                    
                    // update last execution date
                    $lastexecuted = $today;
                    $sql = "UPDATE ".$cf_db_prefix."standingorder "
                        ."SET lastexecuted='".$today."',"
                        ."nextexecution='".$nextexecution."'"
		        . "WHERE soid='".$so_execute->Record['soid']."'";
                    $so_update->query($sql);
                    if($so_update->errno != 0)
                    {
                        utgn_trace("so_execute", $so_execute->getError());
                    }
                    $noso++;
                }
            }
        }
        
        return $noso;  
    }

    /**
     * Converts unix timestamp to database timestamp
     *
     * @access private
     * @param unix time in seconds since 1970
     * @return database timestamp as string string YYYYMMDDhhmmsst
     */
    function UNIX2DBTimestamp($unixtime) {
        return date('Ymdhis',$unixtime);
    }
    
    /**
     * Converts datbase timestamp format to unix timestamp to unix time
     *
     * @access private
     * @param dbtime time as string string YYYYMMDDhhmmsst
     * @return time in seconds since 1970
     */
    function DB2UNIXTimestamp($dbtime) {
        $ts = mktime(
            substr($dbtime,8,2),
            substr($dbtime,10,2),
            substr($dbtime,12,2),
            substr($dbtime,4,2),
            substr($dbtime,6,2),
            substr($dbtime,0,4));
    }
}
?>