<?php
/**
 * User business object: user administration
 *
 * $Source: /cvsroot/minbank/minba/user/bo/usbo_user.php,v $
 * $Author: mboller $
 * $Date: 2006/01/16 01:23:03 $
 * $Revision: 1.3 $
 */

$appl_path=dirname(__FILE__)."/..";
require_once("$appl_path/usgn_config.php");
require_once("$appl_path/bo/usbo_security.php");
require_once("$minsoft_path/utility/utdb_access.php");
require_once("$minsoft_path/utility/utgn_checkit.php");
require_once("$minsoft_path/utility/utrl_role.php");
require_once("$minsoft_path/utility/utgn_trace.php");
require_once("$minsoft_path/utility/utgn_fault.php");
require_once("$minsoft_path/utility/utgn_message.php");

// administrate users
class User
{        
    /**
     * Reads the user.
     * @param $secId secutity id
     * @param $lang language for messages
     * @param $userid id of user
     * @return array of user attributes or fault
     */
    function read($secId, $lang, $userid)
    {
        global $uscf_db_prefix;

        // set language for messages
        $cf_lang = $lang;

        // check for valid security id
        $sec = checkSecurity($secId);
        
        if(isFault($sec))
            return $sec; 
        
        // check if user is system administrator
        if(! ($sec['role'] & utrl_sysadmin))
            return getFault(200);
        
        // read user from database
        $db = &new DBAccess;
        $sql = "SELECT userid, username, usermail, role ".
                "FROM {$uscf_db_prefix}user ".
                "WHERE userid='$userid'";
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);
        
        // check if user exists
        if(! $db->next_record()) {
            print($sql);
                    return getFault(207);
            }        
        // return user data
        $ret = array(
                'userid'=>$db->Record['userid'],
                'username'=>$db->Record['username'],
                'usermail'=>$db->Record['usermail'],
                'role'=>$db->Record['role']);
        return $ret;
    }
    
    /**
     * Adds a user.
     * @param $secId secutity id
     * @param $lang language for messages
     * @param $user array of user attributes
     * @return fault or nothing
     */
    function add($secId, $lang, $user)
    {
        global $uscf_db_prefix;
        // set language for messages
        $cf_lang = $lang;

        // check for valid rights
        $sec = checkSecurity($secId);        
        if(isFault($sec))
            return $sec; 
        if(! ($sec['role'] & utrl_sysadmin))
            return getFault(200);
        
        // check if user already exists    
        if($this->pcheck($user['userid']))
            return getFault(201);
        
        // add user to database
        $db = &new DBAccess;
        $sql = "INSERT INTO {$uscf_db_prefix}user ".
                "(userid, username, userpwd, usermail, role ) ".
                "VALUES (".
                    "'{$user['userid']}',".
                    "'{$user['username']}',".
                    "'{$user['userpwd']}',".
                    "'{$user['usermail']}',".
                    "'{$user['role']}')";
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);

        return 0;        
    }

    /**
     * Updates a user.
     * @param $secId secutity id
     * @param $lang language for messages
     * @param $user array of user attributes
     * @return fault or nothing
     */
    function update($secId, $lang, $user)
    {
        global $uscf_db_prefix;

        // set language for messages
        $cf_lang = $lang;
        
        // check for valid rights
        $sec = checkSecurity($secId);        
        if(isFault($sec))
            return $sec; 
        if(! ($sec & utrl_sysadmin))
            return getFault(200);
            
        // check if user exists
        if(! $this->pcheck($user['userid']))
            return getFault(207);

        // save changes to database
        $db = &new DBAccess;
        $sql = "UPDATE {$uscf_db_prefix}user ".
            "SET ".
                "username='{$user['username']}',".
                "usermail='{$user['usermail']}',".
                "role='{$user['role']}' ".
            "WHERE userid='{$user['userid']}'";        
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);

        // return OK
        return 0;
    }
    
    /**
     * Deletes a user.
     * @param $secId secutity id
     * @param $lang language for messages
     * @param $userid id of user
     * @return fault or 0
     */
    function del($secId, $lang, $userid)
    {
        global $uscf_db_prefix;

        // set language for messages
        $cf_lang = $lang;
        
        // check for valid rights
        $sec = checkSecurity($secId);        
        if(isFault($sec))
            return $sec; 
        if(! ($sec['role'] & utrl_sysadmin))
            return getFault(200);

        // check if user exists
        if(! $this->pcheck($userid))
            return getFault(207);

        // delete user from database
        $db = &new DBAccess;
        $sql = "DELETE FROM {$uscf_db_prefix}user ".
                "WHERE userid='$userid'";
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);

        // return OK
        return 0;
    }

    /**
     * Lists all users.
     * @param $secId secutity id
     * @param $lang language for messages
     * @return array of arrays of user attributes
     */
    function listAll($secId, $lang)
    {
        global $uscf_db_prefix;

        // set language for messages
        $cf_lang = $lang;
        
        // check for valid rights
        $sec = checkSecurity($secId);
        if(isFault($sec))
            return $sec; 
        if(! ($sec['role'] & utrl_sysadmin))
            return getFault(200);

        // read list from database
        $db = &new DBAccess;
        $sql = "SELECT userid, username, role ".
                "FROM {$uscf_db_prefix}user ".
                "ORDER BY userid";
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);
        
        // convert to return array
        $list = array();
        while($db->next_record())
        {
            $list[] = array(
                'userid'=>$db->Record['userid'],
                'username'=>$db->Record['username'],
                'role'=>(int) $db->Record['role']);
        }
        
        // return result
        return $list;
    }
    
    /**
     * Changes the password of a user.
     * @param $secId secutity id
     * @param $lang language for messages
     * @param $userid id of user
     * @param $newPassword new password
     * @return fault or 0
     */
    function changePassword($secId, $lang, $userid, $newPassword)
    {
        global $uscf_db_prefix;

        // set language for messages
        $cf_lang = $lang;
        
        // check for valid rights
        $sec = checkSecurity($secId);        
        if(isFault($sec))
            return $sec; 
            
        $secuserid = $sec['userid'];

        if($secuserid != $userid && !($sec['role'] & utrl_sysadmin))
            return getFault(200);
             
        // update password        
        $db = &new DBAccess;
        $sql = "UPDATE {$uscf_db_prefix}user ".
            "SET userpwd='$newPassword' ".
            "WHERE userid='$userid'";
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);
        
        // return OK
        return 0;
    }
    
    /**
     * Makes a login and returns security id.
     * @param $lang language for messages
     * @param $userid id of user
     * @param $userpwd password of user
     * @return security id or fault
     */
    function login($lang, $userid, $userpwd)
    {
        global $uscf_db_prefix;
        
        // check for valid userid
        if($userid == "")
            return getFault(204);
        else
        {
            if(!checkit::isAlphaNum($userid,TRUE))
                return getFault(203);
        }
           
        // check for valid password
        if($userpwd == "")
             return getFault(205);
        
        // get user
        $db = &new DBAccess;
        $sql = "SELECT username,userpwd,role ".
                "FROM {$uscf_db_prefix}user ".
                "WHERE userid='$userid'";
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);
        
        // check if user exists
        if(! $db->next_record())
            return getFault(207);
        
        // check password
        if($db->Record["userpwd"] != $userpwd)
            return getFault(206);    
        
        // build security id
        $secId = $userid.":".$userpwd;
        
        // return role and security id
        $ret = array($db->Record["role"],$secId);
        return $ret;
        
    } 

    //-----------------------------------------------------
    // private functions
    
    /**
     * Checks existence of a user id.
     * @access private
     * @param $userid user id
     * @return true=user exists
     */
    function pcheck($userid)
    {
        global $uscf_db_prefix;

        // try to read user
        $db = &new DBAccess;
        $sql = "SELECT username ".
                "FROM {$uscf_db_prefix}user ".
                "WHERE userid='$userid'";
        $db->query($sql);
        if($db->errno != 0)
            return getFault(50,$db->msg);
            
        // return true if record exists
        return $db->next_record();
    }
}
?>