<?php

require_once ("$minsoft_path/utility/utgn_trace.php");
require_once ("$minsoft_path/utility/utgn_message.php");

global $mbrsMethods;

// encode an xmlrpc request
function xmlrpc_encode_request( $method, $params)
{
    global $cf_lang;
    
    $request =
            "<?xml version='1.0' encoding='iso-8859-1' ?>\n".
            "<methodCall>\n".
            "<methodName>".$method."</methodName>\n".
            "<params>\n";
            
    if(is_array($params))
    {
        foreach($params as $param)
        {
            if( ! isset($param) )
                return getFault($cf_lang, 101);
            $request .= "<param><value>".
                    mbrpc_encode_var($param).
                    "</value></param>";
        }
    }
    else
    {
        if( ! isset($params) )
            return getFault($cf_lang, 101);
        $request .= "<param>".
                mbrpc_encode_var($params).
                "</param>";    
    }
    
    $request .=
            "</params>\n".
            "</methodCall>\n";

    return $request;
}

// encode an xmlrpc response
function xmlrpc_encode ($params)
{
    $paramsEnc = "";
    
    if(is_array($params))
    {
        if(array_key_exists("faultCode",$params) &&
                array_key_exists("faultString",$params))
        {
            $response =
                    "<?xml version='1.0' encoding='iso-8859-1' ?>\n".
                    "<methodResponse>\n".
                    "  <fault>\n".
                    "    <value>\n".
                    mbrpc_encode_var($params).
                    "    </value>\n".
                    "  </fault>\n".
                    "</methodResponse>\n";
            return $response;
        }
    }
        
    $response =
            "<?xml version='1.0' encoding='iso-8859-1' ?>\n".
            "<methodResponse>\n".
            "  <params>\n".
            "    <param>\n".
            "      <value>\n".
            mbrpc_encode_var($params).
            "      </value>\n".
            "    </param>\n".
            "  </params>\n".
            "</methodResponse>\n";
            
    return $response;
}

function xmlrpc_decode_request($request, &$method, $encoding="")
{    
    // XML-Daten parsen
    $parser = &new MBRPCParser;
    $parser->parse($request);
    $method = $parser->getMethod();
    
    return $parser->getParams();
}

function xmlrpc_server_create()
{
    return 1;
}
    
function xmlrpc_server_destroy($server)
{
    global $mbrsMethods;
    $mbrsMethods = '';
}
    
function xmlrpc_server_register_method(
    $server,
    $method_name,
    $function)
{
    global $mbrsMethods;
    $mbrsMethods[$method_name] = $function;
}
    
function xmlrpc_server_call_method(
    $server,
    $xml,
    $user_data,
    $output_options)
{
    global $mbrsMethods;

    $parser =&new MBRPCParser;
    $parser->parse($xml);
    $result = $mbrsMethods[$parser->getMethod()](
            $parser->getMethod(),
            $parser->getParams(),
            $user_data);
            
    return xmlrpc_encode($result);
}

class MBRPCParser
{
    var $arrayPos;      // actual position in array
    var $currentType;   // current element type
    var $methodName;    // name of method
    var $name;          // actual structure member name
    var $params;        // parameter array
    var $states;        // state stack
    var $vars;            // variable stack
    var $is_response;   // flag that it is a response 

    function MBRPCParser()
    {
        if(! defined("inArray"))
            define("inArray", 1);
        if(! defined("inStruct"))
            define("inStruct", 2);
        $this->methodName = '';    
    }
    
    function parse($response)
    {
        // initialize
        $this->params = array();        
        $this->states = array();
        array_push($this->states, inArray);
        $this->vars = array();
        array_push($this->vars, & $this->params);
        
        // start parsing
        $parser=xml_parser_create();
        xml_set_object($parser, &$this); 
        xml_set_element_handler( $parser, 'start_element', 'end_element');
        xml_set_character_data_handler( $parser, 'client_data');
        xml_parse($parser, $response);
        if($this->is_response && count($this->params) == 1)
            $this->params = $this->vars[0][0];    
        else if(count($this->params) == 0)
            $this->params = null;
        else
            $this->params = $this->vars[0];
        
        xml_parser_free($parser);
    }
    
    function getMethod()
    {
        return $this->methodName;
    }
    
    function getParams()
    {
        return $this->params;
    }
    
    function start_element($parser, $element, $attributes)
    {
        $element = strtoupper($element);
        switch($element) {
        case 'METHODCALL':
            $this->is_response = FALSE;
            $this->currentType = 'NONE';
            break;
        case 'METHODRESPONSE':
            $this->is_response = TRUE;
            $this->currentType = 'NONE';
            break; 
        case 'PARAMS': 
        case 'PARAM':
            $this->currentType = 'NONE';
            break;
        case 'NAME':
            $this->currentType = 'NAME';
            break;
        case 'VALUE':
            if(end($this->states) == inArray)
                $this->arrayPos = count(end($this->vars));
            $this->currentType = 'NONE';
            break;
        case 'ARRAY':
        case 'STRUCT':
        case 'FAULT':
            $top = & $this->vars[count($this->vars)-1];
            if(end($this->states) == inStruct)
            {
                $top[$this->name] = array();
                array_push($this->vars, & $top[$this->name]);
            } else {
                $top[] = array();
                array_push($this->vars, & $top[count($top)-1]);
            }
            if($element =='STRUCT') 
                array_push($this->states,inStruct);
            else
                array_push($this->states,inArray);
            $this->currentType = 'NONE';
            break;
        case 'STRING':
            $this->currentType = 'STRING';
            break;
        case 'I4':
        case 'INT':
            $this->currentType = 'INT';
            break;
        case 'DOUBLE':
            $this->currentType = 'DOUBLE';
            break;
        case 'BOOLEAN':
            $this->currentType = 'BOOLEAN';
            break;
        case 'DATETIME.ISO8601':
            $this->currentType = 'DATETIME';
            break;
        case 'BASE64':
            $this->currentType = 'BASE64';
            break;        
        case 'METHODNAME':
            $this->currentType = 'METHODNAME';
            break;
        default:
            $this->currentType = 'NONE';
            break;
        }    
    }

    function end_element($parser, $element)
    {
        switch($element) {
        case 'STRUCT':
        case 'ARRAY':
            array_pop($this->vars);
            array_pop($this->states);
            break;
        default:
            break;
        }
        $this->currentType='NONE';    
    }

    function client_data($p, $text)
    {
        switch($this->currentType) {
            case 'METHODNAME':
                $this->methodName = $text;
                break;
            case 'NAME':
                $this->name = $text;
                $top = & $this->vars[count($this->vars)-1];
                $top[$this->name] = "";
                break;
            case 'NONE':
                break;
            default:
                $top = & $this->vars[count($this->vars)-1];
                // beware: expat separates values at ampersands, so I recatenate
                // beware: expat converts entries i.e. &quot; to their corresponding characters
                if(end($this->states) == inArray)
                {
                    if(count($top) > $this->arrayPos)
                        $top[$this->arrayPos] .= htmlspecialchars($text);
                    else
                        $top[$this->arrayPos] = htmlspecialchars($text);
                }
                else
                {
                    if(array_key_exists($this->name,$top))
                        $top[$this->name] .= htmlspecialchars($text);
                    else
                        $top[$this->name] = htmlspecialchars($text);
                }
        }
    }
}

function mbrpc_encode_var($var)
{
    if(is_string($var)) {
        return "<string>".htmlspecialchars($var)."</string>\n";
    } else if(is_bool($var)) {
        return "<boolean>$var</boolean>\n";
    } else if(is_float($var)) {
        return "<double>$var</double>\n";
    } else if(is_int($var)) {
        return "<int>$var</int>\n";
    } else if(is_array($var)) {
        if(array_key_exists(0, $var))
        {
            $result = "<array>\n".
                    "<data>\n";
            foreach($var as $element)
            {
                $result .= "<value>\n".mbrpc_encode_var($element)."</value>\n";
            }
            return $result."</data>\n</array>\n";
        } else {
            $result="<struct>\n";
            foreach($var as $key=>$value)
            {
                $result .=
                        "<member>\n".
                        "  <name>$key</name>\n".
                        "  <value>\n".
                        mbrpc_encode_var($value).
                        "  </value>\n".
                        "</member>\n";
            }
            return $result."</struct>\n";
        }
    }
}

?>