<?php

require_once("$minsoft_path/utility/utgn_config.php");
require_once("$minsoft_path/utility/utgn_message.php");
require_once("$minsoft_path/utility/utgn_trace.php");

// if the XML-RPC-Extension of PHP does not exist,
// we will load our own emulation of this interface. 
// For the current version the expat parser has to be available.
if(! extension_loaded('xmlrpc'))
{
    include_once("$minsoft_path/utility/utxr_epiemu.php");
}

// catch an xml-rpc call
//
// xml-rpc-specific error handling is initialized
// the post data is fetched
// the xml-rpc-server is initialized using the given method list
// the method is called and the response returned

function utxr_catch($rpcMethods,$programMethods)
{
    if(! headers_sent() )
    {
        header('Content-Type: text/xml');    
    }
    
    // set the error reporting
    error_reporting (E_ALL & ~ E_NOTICE);

    // set to the error handler
    $old_error_handler = set_error_handler("utxr_errorHandler");

    // get posted data
    $data = $GLOBALS["HTTP_RAW_POST_DATA"];

    // remove backslashes
    if(get_magic_quotes_gpc())
    {
        $data =
            stripslashes ( preg_replace("/\\\'/","\\\"",$data) );
    }

    // create server
    $server = xmlrpc_server_create();

    if($server)
    {
        for($i=0; $i < count($rpcMethods); $i++)
        {
            // register methods at server
            xmlrpc_server_register_method($server, $rpcMethods[$i], $programMethods[$i]);
        }
        
        // set header to xml
        if( ! headers_sent() )
        {
            header("Connection: close");
        }
        
        // set output to xml
        $output_options = array("output_type" => "xml");
        // utgn_trace("catch", "DATA=".$data);
        
        // call method supplied by client
        $response = xmlrpc_server_call_method($server, $data, NULL, $output_options);
        
        // return length of data
        if( ! headers_sent() )
        {
            header("Content-Length: ", strlen($response));
        }
        
        // return data
        echo $response;
    }
    else
    {
        trigger_error('Can not create xmlrpc server', E_ERROR);
    }
    
    return 0;
}

// error handler function
function utxr_errorHandler ($errno, $errstr, $errfile, $errline)
{
    global $utcv_error_log;
    global $utcv_error_path;
    
    $errorText = "ERROR: ".$errno." in ".$errfile." at ".$errline.":".$errstr;
    
    // write error to error logging file
    if(isset($utcv_error_log) && $utcv_error_log = '1' && $utcv_error_path != "")
    {
        error_log(date("Y-m-d H:i:s | ").$errorText."\r\n", 3,
            $utcv_error_path);
    }
    
    // do error type specific handling
    switch ($errno) {
    case E_PARSE:
    case E_CORE_ERROR:
    case E_COMPILE_ERROR:
        // these will not work because of PHP limitations
        utxr_displayResponse(getFault(2, 1,$errorText));
        exit(1);
    case E_CORE_WARNING:
    case E_COMPILE_WARNING:
        utxr_displayResponse(getFault(2, 2,$errorText));
        exit(1);    
    case E_ERROR:
        utxr_displayResponse(getFault(2, 3,$errorText));
    case E_WARNING:
        utxr_displayResponse(getFault(2, 4,$errorText));
        exit(0);
    case E_NOTICE:
        return;
        // should be activated during first tests
        utxr_displayResponse(getFault(2, 5,$errorText));
        exit(0);
    case E_USER_ERROR:
        utxr_displayResponse(getFault(2, 6,$errorText));
        exit(1);
    case E_USER_WARNING:
        utxr_displayResponse(getFault(2, 7,$errorText));
        exit(0);
    case E_USER_NOTICE:
        return;
    default:
        utxr_displayResponse(getFault(2, 8,$errorText));
        exit(0);
    }
    exit(0);
}

// return an php datastructure via xml-rpc
function utxr_displayResponse($returnValue)
{
    $response = xmlrpc_encode($returnValue);
    if( ! headers_sent() )
    {
        header("Content-Length: ", strlen($response));
    }
    echo $response;
}
?>