<?php

global $mbrsMethods;

/**
 * Class for simplified working as a XML-RPC-client
 */ 
class UTXRClient
{
    var $server;
    var $path;

    function UTXRClient() {
        global $safe_mode, $minsoft_path;

        $this->server = '';
        $this->path = '';
    }
    
    function setServer($server, $path, $port=80)
    {
        $this->server = $server;
        $this->path = $path;
    }
    
    function call($function, $params)
    {
        // check if initialized
        if($this->server == '')
        {
            $result = getFault(2, 100);
            return $result;
        }
                
        // echo "PARAM=".utgn_dump($params);    
        
        // code request to xml-rpc
        $request = minba_xmlrpc_encode_request( $function, $params);
        
        // open http-socket
        $fp = fsockopen($this->server, 80, $errno, $errstr, 30);
        if (!$fp)
        {
            $result = $this->server.": $errstr ($errno)";
            return $result;
        }

        // send request using  http
        $send = "POST {$this->path} HTTP/1.1\r\n".
            "User-Agent: PHP XMLRPC 1.0\r\n".
            "Host: {$this->server}\r\n".
            "Content-type: text/xml\r\n".
            "Content-length: ".strlen($request)."\r\n\r\n".
            "$request";
            
        // echo "REQUEST=".$send;
        
        fputs($fp, $send);
        
        // get result
        $xmlresult = '';
        // socket_set_timeout($fp, 10); // sometimes not supported
        while(!feof($fp)) {
            $line = fgets($fp, 256);
            $status = socket_get_status($fp);
            if($status['timed_out'])
            {
                $result = array(array("faultCode" => 9999, 
                        "faultString" => "Connection Timeout".$xmlresult));
                fclose($fp);
                return;
            }
            $xmlresult .= $line;
        }
        // close socket
        fclose($fp);
        
        // echo "RESPONSE=".$xmlresult;
        
        // decode result
        if ( preg_match("/^HTTP\/[0-9\.]+ 200 /i" ,$xmlresult) )
        {
            preg_match("/(^HTTP\/.*)(\<\?xml.*$)/si",
                    $xmlresult, $matches);    

            if(is_array($matches) && count($matches) > 2 && $matches[2] != "")
            {
                $result = minba_xmlrpc_decode_request($matches[2], &$method);
                // echo "RESULT=".utgn_dump($result);
            }
            else
            {
                $result = array(array("faultCode" => 9999, "faultString" => $xmlresult));
            }
        }
        else
        {
            $result = array(array("faultCode" => 9999, "faultString" => $xmlresult));
        }
        
        // echo utgn_dump($result);
        return $result;
    }
}

/**
 * Encodes a XML-RPC request.
 *
 * @param $method name of the method to be called.
 * @param $params array of parameters
 * @return XML-RPC request or fault
 */
function minba_xmlrpc_encode_request( $method, $params)
{
    global $cf_lang;
    
    $request =
            "<?xml version='1.0' encoding='iso-8859-1' ?>\n".
            "<methodCall>\n".
            "<methodName>".$method."</methodName>\n".
            "<params>\n";
            
    if(is_array($params))
    {
        foreach($params as $param)
        {
            if( ! isset($param) )
                return makeFault(101, "parameter not initialized");
            $request .= "<param><value>".
                    mbrpc_encode_var($param).
                    "</value></param>";
        }
    }
    else
    {
        if( ! isset($params) )
            return makeFault(101, "parameter not initialized");
        $request .= "<param>".
                mbrpc_encode_var($params).
                "</param>";    
    }
    
    $request .=
            "</params>\n".
            "</methodCall>\n";

    return $request;
}

/**
 * Encodes a XML-RPC response.
 *
 * @param $params array of parameters
 * @return response
 */
function minba_xmlrpc_encode ($params)
{
    $paramsEnc = "";
    
    if(is_array($params))
    {
        if(array_key_exists("faultCode",$params) &&
                array_key_exists("faultString",$params))
        {
            $response =
                    "<?xml version='1.0' encoding='iso-8859-1' ?>\n".
                    "<methodResponse>\n".
                    "  <fault>\n".
                    "    <value>\n".
                    mbrpc_encode_var($params).
                    "    </value>\n".
                    "  </fault>\n".
                    "</methodResponse>\n";
            return $response;
        }
    }
        
    $response =
            "<?xml version='1.0' encoding='iso-8859-1' ?>\n".
            "<methodResponse>\n".
            "  <params>\n".
            "    <param>\n".
            "      <value>\n".
            mbrpc_encode_var($params).
            "      </value>\n".
            "    </param>\n".
            "  </params>\n".
            "</methodResponse>\n";
            
    return $response;
}


function minba_xmlrpc_decode_request($request, &$method, $encoding="")
{    
    // XML-Daten parsen
    $parser = &new MBRPCParser;
    $parser->parse($request);
    $method = $parser->getMethod();
    
    return $parser->getParams();
}

function minba_xmlrpc_server_create()
{
    return 1;
}
    
function minba_xmlrpc_server_destroy($server)
{
    global $mbrsMethods;
    $mbrsMethods = '';
}
    
function minba_xmlrpc_server_register_method(
    $server,
    $method_name,
    $function)
{
    global $mbrsMethods;
    $mbrsMethods[$method_name] = $function;
}

/**
 * Calls a method on a server using XML-RPC.
 *
 * @param $server Server providing the method.
 * @param $xml
 */   
function minba_xmlrpc_server_call_method(
    $server,
    $xml,
    $user_data,
    $output_options)
{
    global $mbrsMethods;

    $parser =&new MBRPCParser;
    $parser->parse($xml);
    $result = $mbrsMethods[$parser->getMethod()](
            $parser->getMethod(),
            $parser->getParams(),
            $user_data);
            
    return xmlrpc_encode($result);
}

class MBRPCParser
{
    var $arrayPos;      // actual position in array
    var $currentType;   // current element type
    var $methodName;    // name of method
    var $name;          // actual structure member name
    var $params;        // parameter array
    var $states;        // state stack
    var $vars;            // variable stack
    var $is_response;   // flag that it is a response 

    function MBRPCParser()
    {
        if(! defined("inArray"))
            define("inArray", 1);
        if(! defined("inStruct"))
            define("inStruct", 2);
        $this->methodName = '';    
    }
    
    function parse($response)
    {
        // initialize
        $this->params = array();        
        $this->states = array();
        array_push($this->states, inArray);
        $this->vars = array();
        array_push($this->vars, & $this->params);
        
        // start parsing
        $parser=xml_parser_create();
        xml_set_object($parser, &$this); 
        xml_set_element_handler( $parser, 'start_element', 'end_element');
        xml_set_character_data_handler( $parser, 'client_data');
        xml_parse($parser, $response);
        if($this->is_response && count($this->params) == 1)
            $this->params = $this->vars[0][0];    
        else if(count($this->params) == 0)
            $this->params = null;
        else
            $this->params = $this->vars[0];
        
        xml_parser_free($parser);
    }
    
    function getMethod()
    {
        return $this->methodName;
    }
    
    function getParams()
    {
        return $this->params;
    }
    
    function start_element($parser, $element, $attributes)
    {
        $element = strtoupper($element);
        switch($element) {
        case 'METHODCALL':
            $this->is_response = FALSE;
            $this->currentType = 'NONE';
            break;
        case 'METHODRESPONSE':
            $this->is_response = TRUE;
            $this->currentType = 'NONE';
            break; 
        case 'PARAMS': 
        case 'PARAM':
            $this->currentType = 'NONE';
            break;
        case 'NAME':
            $this->currentType = 'NAME';
            break;
        case 'VALUE':
            if(end($this->states) == inArray)
                $this->arrayPos = count(end($this->vars));
            $this->currentType = 'NONE';
            break;
        case 'ARRAY':
        case 'STRUCT':
        case 'FAULT':
            $top = & $this->vars[count($this->vars)-1];
            if(end($this->states) == inStruct)
            {
                $top[$this->name] = array();
                array_push($this->vars, & $top[$this->name]);
            } else {
                $top[] = array();
                array_push($this->vars, & $top[count($top)-1]);
            }
            if($element =='STRUCT') 
                array_push($this->states,inStruct);
            else
                array_push($this->states,inArray);
            $this->currentType = 'NONE';
            break;
        case 'STRING':
            $this->currentType = 'STRING';
            break;
        case 'I4':
        case 'INT':
            $this->currentType = 'INT';
            break;
        case 'DOUBLE':
            $this->currentType = 'DOUBLE';
            break;
        case 'BOOLEAN':
            $this->currentType = 'BOOLEAN';
            break;
        case 'DATETIME.ISO8601':
            $this->currentType = 'DATETIME';
            break;
        case 'BASE64':
            $this->currentType = 'BASE64';
            break;        
        case 'METHODNAME':
            $this->currentType = 'METHODNAME';
            break;
        default:
            $this->currentType = 'NONE';
            break;
        }    
    }

    function end_element($parser, $element)
    {
        switch($element) {
        case 'STRUCT':
        case 'ARRAY':
            array_pop($this->vars);
            array_pop($this->states);
            break;
        default:
            break;
        }
        $this->currentType='NONE';    
    }

    function client_data($p, $text)
    {
        switch($this->currentType) {
            case 'METHODNAME':
                $this->methodName = $text;
                break;
            case 'NAME':
                $this->name = $text;
                $top = & $this->vars[count($this->vars)-1];
                $top[$this->name] = "";
                break;
            case 'NONE':
                break;
            default:
                $top = & $this->vars[count($this->vars)-1];
                // beware: expat separates values at ampersands, so I recatenate
                // beware: expat converts entries i.e. &quot; to their corresponding characters
                if(end($this->states) == inArray)
                {
                    if(count($top) > $this->arrayPos)
                        $top[$this->arrayPos] .= htmlspecialchars($text);
                    else
                        $top[$this->arrayPos] = htmlspecialchars($text);
                }
                else
                {
                    if(array_key_exists($this->name,$top))
                        $top[$this->name] .= htmlspecialchars($text);
                    else
                        $top[$this->name] = htmlspecialchars($text);
                }
        }
    }
}

function mbrpc_encode_var($var)
{
    if(is_string($var)) {
        return "<string>".htmlspecialchars($var)."</string>\n";
    } else if(is_bool($var)) {
        return "<boolean>$var</boolean>\n";
    } else if(is_float($var)) {
        return "<double>$var</double>\n";
    } else if(is_int($var)) {
        return "<int>$var</int>\n";
    } else if(is_array($var)) {
        if(array_key_exists(0, $var))
        {
            $result = "<array>\n".
                    "<data>\n";
            foreach($var as $element)
            {
                $result .= "<value>\n".mbrpc_encode_var($element)."</value>\n";
            }
            return $result."</data>\n</array>\n";
        } else {
            $result="<struct>\n";
            foreach($var as $key=>$value)
            {
                $result .=
                        "<member>\n".
                        "  <name>$key</name>\n".
                        "  <value>\n".
                        mbrpc_encode_var($value).
                        "  </value>\n".
                        "</member>\n";
            }
            return $result."</struct>\n";
        }
    }
}

/**
 * Makes a fault structure from error number and error text
 *
 * @param $number error number
 * @param $text error text
 * @return fault object
 */
function makeFault($number, $text) {
    return array(array("faultCode" => $number, "faultString" => $text));
}

/**
 * Checks if a response is a fault structure
 *
 * @param $response response to be checked
 * @return true=it is a fault
 */
function isFault($response)
{
    if(is_array($response) && array_key_exists(0, $response))
    {
        if(is_array($response[0]))
        {
            if(array_key_exists("faultCode",$response[0]) &&
                    array_key_exists("faultString",$response[0]))
            {
                return TRUE;
            }
        }
    }        
    return FALSE;
}
?>